#!/usr/bin/env python
"""Common decorators for integration in generic views."""

from functools import wraps
from notmm.utils.wsgilib.exc import HTTPForbiddenResponse

def post_required(view_func, **kwargs):
    def _wrapper(*args, **kwargs):
        req = args[0]
        if (req.method != 'POST'):
            return HTTPForbiddenResponse("Please use POST!", status=403, mimetype='text/plain')
        return view_func(req, *args, **kwargs)
    return wraps(view_func)(_wrapper, **kwargs)



# Suck in default django settings
from django.conf.global_settings import *

from pkg_resources import resource_filename

import os

DEBUG = True

# Keep this to False unless you expecting to debug I18N
USE_I18N = False

SECRET_KEY = 'secretkey'

# Custom error handlers mapping
CUSTOM_ERROR_HANDLERS = (
    #('handle401', 'sandbox.handlers.handle401'),
    #('handle403', 'sandbox.handlers.handle403'),
    ('handle404', 'sandbox.handlers.handle404'),
    ('handle500', 'sandbox.handlers.handle500')
    )

# List of processors used by RequestContext to populate the context.
# Each one should be a callable that takes the request object as its
# only parameter and returns a dictionary to add to the context.
TEMPLATE_CONTEXT_PROCESSORS = (
   'django.core.context_processors.request', # request
   'django.core.context_processors.media',
   'blogengine.contrib.addthis.context_processors.addthis_widget',  # addthis widget
   'django.contrib.auth.context_processors.auth'
   )
# For backward-compatibility with Django
TEMPLATE_DIRS = (
    resource_filename('sandbox', 'templates'),
)

ROOT_URLCONF = 'sandbox.config.urls'

MIDDLEWARE_CLASSES = ()

#SITE_ID = 1

#    )
INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.contenttypes'
    )
ENABLE_TIDYLIB = False
ENABLE_LOGGING = True
LOGGING_CALLBACK = 'rootLogger'

SCHEVO_DATABASE_NAME = 'schevo.db.blogs'

ADDTHIS_USERNAME = 'erob'

CACHE_BACKEND_LOCATION = '127.0.0.1:11221'



# Copyright (C) 2005 Red Hat, Inc.
# Copyright (C) 2006 Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from gazpacho import gapi, util
from gazpacho.command import Command
from gazpacho.clipboard import CommandCutPaste
from gazpacho.commandmanager import command_manager
from gazpacho.gadget import Gadget
from gazpacho.i18n import _
from gazpacho.placeholder import Placeholder
from gazpacho.widgets.base.box import DragAppendCommand, CreateAppendCommand, \
     DragExtendCommand, CreateExtendCommand

(DND_POS_INVALID,
 DND_POS_TOP,
 DND_POS_BOTTOM,
 DND_POS_LEFT,
 DND_POS_RIGHT,
 DND_POS_BEFORE,
 DND_POS_AFTER) = list(range(7))

(INFO_TYPE_XML,
 INFO_TYPE_WIDGET,
 INFO_TYPE_PALETTE) = list(range(3))

MIME_TYPE_OBJECT_XML = 'application/x-gazpacho-object-xml'
MIME_TYPE_OBJECT     = 'application/x-gazpacho-object'
MIME_TYPE_OBJECT_PALETTE = 'application/x-gazpacho-palette'

# This is the value returned by Widget.drag_dest_find_target when no
# targets have been found. The API reference says it should return
# None (and NONE) but it seems to return the string "NONE".
DND_NO_TARGET = "NONE"

DND_WIDGET_TARGET  = (MIME_TYPE_OBJECT, gtk.TARGET_SAME_APP, INFO_TYPE_WIDGET)
DND_XML_TARGET     = (MIME_TYPE_OBJECT_XML, 0, INFO_TYPE_XML)
DND_PALETTE_TARGET = (MIME_TYPE_OBJECT_PALETTE, gtk.TARGET_SAME_APP,
                      INFO_TYPE_PALETTE)

class DnDHandler(object):
    """
    The DnDHandler is responsible for handling events related to
    moving or copying widget by drag and drop.

    It might be worth mentioning a few things about the naming
    convention used in the implementation. This mainly concern the
    widgets that are participating in the drag and drop related
    activities.

    First we have the source widget. This is the widget that is being
    dragged and it is the top-most (non-internal and non-hidden)
    widget that has been clicked on. Unfortunately not all widgets can
    receive the necessary drag/drop events and thus another widget
    might be the one that received these events. This widget is
    referred to as the source event widget. The source event widget
    will calculate the source widget and keep a reference to it.

    Then there is the target widget. This is the widget onto which the
    source widget will be dropped. As with the source widget, the
    target widget might not be able to receive the drag/drop events
    and thus another widget will receive them in its place. This
    widget is referred to as the target event widget. The target
    widget has to be calculated when it is needed.

    If it will be clear from the context, the source event widget and
    target event widget will sometimes be referred to as the event
    widget.
    """

    # XXX what happens when we leave a target widget and enters a new
    #     target widget when both target widgets are children of the
    #     same target event widget? This would probably only affect
    #     the clearing of the drag highlight, if it matters at all.

    # XXX there seems to be some problems with gtk.Entry. It's
    #      possible to drag it onto itself when clicking on the actual
    #      text entry and dragging it near the edge of the entry. See
    #      _on_widget__drag_data_received for more info

    #
    # Public methods
    #

    def connect_drag_handlers(self, widget):
        """
        Connect all handlers necessary for the widget to serve as a
        drag source.

        @param widget: widget to which the handlers should be connected
        @type widget: gtk.Widget
        """
        widget.drag_source_set(gtk.gdk.BUTTON1_MASK,
                               [DND_WIDGET_TARGET, DND_XML_TARGET],
                               gtk.gdk.ACTION_COPY | gtk.gdk.ACTION_MOVE)
        widget.connect('drag_begin', self._on_widget__drag_begin)
        widget.connect('drag_data_get', self._on_widget__drag_data_get)


    def connect_drop_handlers(self, widget):
        """
        Connect all handlers necessary for the widget to serve as a
        drag destination.

        @param widget: widget to which the handlers should be connected
        @type widget: gtk.Widget
        """
        widget.drag_dest_set(0,
                             [DND_WIDGET_TARGET,
                              DND_XML_TARGET,
                              DND_PALETTE_TARGET],
                             gtk.gdk.ACTION_MOVE | gtk.gdk.ACTION_COPY)
        widget.connect('drag_motion', self._on_widget__drag_motion)
        widget.connect('drag_leave', self._on_widget__drag_leave)
        widget.connect('drag_drop', self._on_widget__drag_drop)
        widget.connect('drag_data_received',
                       self._on_widget__drag_data_received)


    #
    # Methods that can be used and overridden by subclasses
    #

    def _get_target_project(self, widget):
        """
        Get the project to which the widget belongs.

        @param widget: a widget
        @type widget: gtk.Widget
        """
        raise NotImplementedError

    def _set_drag_highlight(self, event_widget, x, y):
        """Highlight the target widget in an appropriate way to
        indicate that this is a valid drop zone.

        @param event_widget: the target event widget
        @type event_widget: gtk.Widget
        @param x: the mouse x position
        @type x: int
        @param y: the mouse y position
        @type y: int
        """
        raise NotImplementedError

    def _clear_drag_highlight(self, event_widget):
        """Clear the drag highligt.

        @param event_widget: the target event widget
        @type event_widget: gtk.Widget
        """
        raise NotImplementedError

    def _is_valid_drop_zone(self, drag_context, target_event_widget, x ,y):
        """Check whether the drop zone is valid or not.

        @param drag_context: the drag context
        @type drag_context: gtk.gdk.DragContext
        @param target_event_widget: the target event widget
        @type target_event_widget: gtk.Widget
        @param x: the X position of the drop
        @type x: int
        @param y: the Y position of the drop
        @type y: int
        @return: whether or not this is a valid drop zone
        @rtype: bool
        """
        # Check if it is a valid target type
        dest_targets = target_event_widget.drag_dest_get_target_list()
        target = target_event_widget.drag_dest_find_target(drag_context,
                                                           dest_targets)
        
        if target == DND_NO_TARGET:
            return False

        # We can always drag widgets from the palette and widgets that
        # are in XML form
        if target in (MIME_TYPE_OBJECT_PALETTE, MIME_TYPE_OBJECT_XML):
            return True

        # Get the correct source widget and target widget
        source_event_widget = drag_context.get_source_widget()
        source_event_gadget = Gadget.from_widget(source_event_widget)
        source_widget = source_event_gadget.dnd_gadget.widget

        target_widget, tx, ty = self._get_target_widget(target_event_widget,
                                                        x, y)

        # We cannot drop onto ourselves
        if source_widget is target_widget:
            return False

        # Make sure we don't try to drop a widget onto one of its
        # children or parents
        if (source_widget.is_ancestor(target_widget)
            or target_widget.is_ancestor(source_widget)):
            return False

        return True

    def _get_drag_action(self, drag_context, target_event_widget):
        """Figure out which drag action should be performed.

        If the target is in the same project as the source we move the
        widget, otherwise we copy it.

        @param drag_context: the drag context
        @type drag_context: gtk.gdk.DragContext
        @param target_event_widget: the target event widget
        @type target_event_widget: gtk.Widget
        """
        # XXX doesn't it make more sense to move only when source and
        #     target is in the same toplevel?
        target_project = self._get_target_project(target_event_widget)
        source_event_widget = drag_context.get_source_widget()
        source_event_gadget = source_event_widget and Gadget.from_widget(source_event_widget)
        if(not source_event_gadget
           or source_event_gadget.project != target_project):
            return gtk.gdk.ACTION_COPY

        return gtk.gdk.ACTION_MOVE

    def _get_source_gadget(self, data, info, drag_context, project):
        """Get the gadget for the source widget.

        This means the gadget for the widget that is being dragged,
        which is not necessarily the same widget as the one that
        recieved the drag event.

        @return: the source gadget
        @rtype: L{gazpacho.gadget.Gadget}
        """
        source_gadget = None
        if info == INFO_TYPE_WIDGET:
            event_widget = drag_context.get_source_widget()
            event_gadget = Gadget.from_widget(event_widget)
            source_gadget = event_gadget.dnd_gadget

        elif info == INFO_TYPE_XML:
            source_gadget = Gadget.from_xml(project, data.data)

        return source_gadget

    def _get_target_widget(self, event_widget, x, y):
        """Get the target widget.

        This means the widget onto which we perform the drop, which is
        not necessarily the same widget as the one that recieved the
        drop event.

        @return: the target widget and translated coordinates
        @rtype: gtk.Widget, int, int
        """
        return event_widget, x, y

    #
    # Signal handlers for the drag source
    #

    def _on_widget__drag_begin(self, event_widget, drag_context):
        """Callback for the 'drag-begin' event."""
        raise NotImplementedError

    def _on_widget__drag_data_get(self, event_widget, drag_context,
                                  selection_data, info, time):
        """Callback for the 'drag-data-get' event."""
        raise NotImplementedError


    #
    # Signal handlers for the drag target
    #

    def _on_widget__drag_leave(self, event_widget, drag_context, time):
        """Clear the drag highlight when we leave the target widget.

        This is a callback for the 'drag-leave' event.
        """
        self._clear_drag_highlight(event_widget)

    def _on_widget__drag_motion(self, event_widget, drag_context, x, y, time):
        """Decide whether this is a valid drop zone or not.

        If the drop zone is valid we set the drag highlight and the
        appropriate drag action. This is a callback for the
        'drag-motion' event.
        """
        if not self._is_valid_drop_zone(drag_context, event_widget, x, y):
            return False

        # XXX: Do not draw highlight if there is a children visible at
        #      position x, y.
        #      Test case is HBox->Notebook, drag notebook into itself, notice
        #      border which is not correct
        self._set_drag_highlight(event_widget, x, y)

        drag_action = self._get_drag_action(drag_context, event_widget)
        drag_context.drag_status(drag_action, time)
        return True

    def _on_widget__drag_drop(self, event_widget, drag_context, x, y, time):
        """Callback for handling the 'drag_drop' event."""
        if MIME_TYPE_OBJECT_PALETTE in drag_context.targets:
            mime_type = MIME_TYPE_OBJECT_PALETTE
        elif drag_context.action == gtk.gdk.ACTION_MOVE:
            mime_type = MIME_TYPE_OBJECT
        else:
            mime_type = MIME_TYPE_OBJECT_XML
        event_widget.drag_get_data(drag_context, mime_type, time)
        return True

    def _on_widget__drag_data_received(self, event_widget, drag_context,
                                       x, y, data, info, time):
        """Callback for the 'drag-data-recieved' event."""
        raise NotImplementedError

class WidgetDnDHandler(DnDHandler):

    def _get_target_widget(self, event_widget, x, y):
        """Get the target widget

        @param event_widget: the target event widget
        @type event: gtk.Widget
        @param x: the X position of the drop
        @type x: int
        @param y: the Y position of the drop
        @type y: int

        @return: the target widget
        @rtype: gtk.Widget
        """
        target_widget = self._find_deepest_child_at_position(event_widget, x, y)
        tx, ty = event_widget.translate_coordinates(target_widget, x, y)
        return target_widget,tx,ty

    def _find_deepest_child_at_position(self, widget, x, y):
        """Find the deepest child the the specified position.

        This method will ignore internal and hidden widgets.

        @return: the deepest widget
        @rtype: gtk.Widget
        """
        # XXX this code is based on Gadget._find_deepest_child_at_position
        # but we need to avoid internal and hidden widgets. Maybe it's possible
        # to merge these two methods?
        found = None
        if isinstance(widget, gtk.Container):
            # we're not interested internal or hidden widgets
            # XXX can these widgets contain children that we
            #     are interested in?            
            # XXX there is no good way to figure out which widgets are
            #     hidden at the moment.
            for child in widget.get_children():
                # hidden widget usually don't have a gadget?
                if not Gadget.from_widget(child):
                    continue
                
                child_x, child_y = widget.translate_coordinates(child, x, y)

                # sometimes the found widget is not mapped or visible
                # think about a widget in a notebook page which is not selected
                if (0 <= child_x < child.allocation.width and
                    0 <= child_y < child.allocation.height and
                    child.flags() & gtk.MAPPED):
                    found = self._find_deepest_child_at_position(child,
                                                                 child_x,
                                                                 child_y)
                    if found:
                        break

        return found or widget

    def _get_target_project(self, widget):
        """
        Get the project to which the target belongs.

        @param widget: the target widget
        @type widget: gtk.Widget
        """
        gadget = Gadget.from_widget(widget)
        return gadget.project

    def _set_drag_highlight(self, event_widget, x, y):
        """
        Highlight the widget in an appropriate way to indicate
        that this is a valid drop zone.

        @param event_widget: the target event widget
        @type event_widget: gtk.Widget
        @param x: the mouse x position
        @type x: int
        @param y: the mouse y position
        @type y: int
        """
        target_widget, tx, ty = self._get_target_widget(event_widget, x, y)
        gadget = Gadget.from_widget(target_widget)
        gadget.set_drop_region(self._get_drop_location(target_widget, tx, ty)[1])

    def _clear_drag_highlight(self, event_widget):  
        """Clear the drag highligt for the target widget.

        This method should initially be called with the target event
        widget as an argument. It will then recursively iterate
        through its own children since we don't actually know which
        whidget is the target widget.

        @param event_widget: the target event widget
        @type event_widget: gtk.Widget
        """
        gadget = Gadget.from_widget(event_widget)
        if gadget:
            gadget.clear_drop_region()

        if isinstance(event_widget, gtk.Container):
            children = util.get_all_children(event_widget)
            for child in children:
                self._clear_drag_highlight(child)


    def _is_valid_drop_zone(self, drag_context, target_event_widget, x ,y):
        """Check whether the drop zone is valid or not.

        This behaves as in the super class but it also checks if we
        are in a valid location (i.e. not in the middle of the widget)

        @param drag_context: the drag context
        @type drag_context: gtk.gdk.DragContext
        @param target_event_widget: the target event widget
        @type target_event_widget: gtk.Widget
        @param x: the X position of the drop
        @type x: int
        @param y: the Y position of the drop
        @type y: int
        @return: whether or not this is a valid drop zone
        @rtype: bool
        """
        if not DnDHandler._is_valid_drop_zone(self, drag_context,
                                              target_event_widget, x, y):
            return False

        # we need to make sure that we actually are in a valid location
        target_widget,tx,ty = self._get_target_widget(target_event_widget,x,y)
        location = self._get_drop_location(target_widget, tx, ty)[0]
        return location != DND_POS_INVALID

    def _get_drop_location(self, target_widget, x, y):
        """Calculate the drop region and also the drop location
        relative to the widget.

        The location can be one of the following constants
          - DND_POS_INVALID
          - DND_POS_TOP
          - DND_POS_BOTTOM
          - DND_POS_LEFT
          - DND_POS_RIGHT
          - DND_POS_BEFORE
          - DND_POS_AFTER

        The drop region is a tuple of x,y,width and height.

        @param target_widget: the widget where the drop occurred
        @type target_widget: gtk.Widget
        @param x: the X position of the drop
        @type x: int
        @param y: the Y position of the drop
        @type y: int

        @return: (location, region)
        @rtype: tuple
        """
        parent = util.get_parent(target_widget)
        h_appendable = parent and isinstance(parent.widget, gtk.HBox)
        v_appendable = parent and isinstance(parent.widget, gtk.VBox)

        x_off, y_off, width, height = target_widget.allocation
        x_third = width / 3
        y_third = height / 3

        if x > x_third * 2:
            if h_appendable:
                location = DND_POS_AFTER
            else:
                location = DND_POS_RIGHT
            region = (x_third * 2 + x_off, 2 + y_off,
                      x_third, height - 4)
        elif x < x_third:
            if h_appendable:
                location = DND_POS_BEFORE
            else:
                location = DND_POS_LEFT
            region = (2 + x_off, 2 + y_off, x_third - 2, height - 4)
        elif y > y_third * 2:
            if v_appendable:
                location = DND_POS_AFTER
            else:
                location = DND_POS_BOTTOM
            region = (2 + x_off, y_third * 2 + y_off, width - 4, y_third - 2)
        elif y < y_third:
            if v_appendable:
                location = DND_POS_BEFORE
            else:
                location = DND_POS_TOP
            region = (2 + x_off, 2 + y_off, width - 4, y_third)
        else:
            location = DND_POS_INVALID
            region = (0, 0, 0, 0)

        return location, region

    def _is_extend_action(self, location):
        """
        Check if we should perform an extend action.

        @param location: the drop location
        @type location: dnd constant
        @return: True if we should extend
        @rtype: bool
        """
        return location in (DND_POS_TOP, DND_POS_BOTTOM, DND_POS_LEFT,
                            DND_POS_RIGHT)

    def _is_append_action(self, location):
        """
        Check if we should perform an append action.

        @param location: the drop location
        @type location: dnd constant
        @return: True if we should append
        @rtype: bool
        """
        return location in (DND_POS_AFTER, DND_POS_BEFORE)

    def _get_append_position(self, target_widget, location):
        """
        Get the position where we should add the widget.

        @param target_widget: the widget where the drop occurred
        @type target_widget: gtk.Widget
        @param location: the drop location
        @type location: dnd constant
        """
        box = target_widget.get_parent()
        pos = box.get_children().index(target_widget)
        if location == DND_POS_AFTER:
            pos = pos + 1
        return pos

    def _execute_drag(self, location, source_gadget, target_gadget, keep_source):
        """
        Execute the drag-append or drag-extend command.

        @param location: the drop location
        @type location: dnd constant
        @param source_gadget: the gadget that is being dragged
        @type source_gadget: L{gazpacho.gadget.Gadget}
        @param target_gadget: the gadget where the drop occurred
        @type target_gadget: L{gazpacho.gadget.Gadget}
        @param keep_source: True if the source widget should not be removed
        @type keep_source: bool

        @return: True if the drop was successful
        @rtype: bool
        """
        if not source_gadget:
            return False

        if self._is_extend_action(location):
            cmd = DragExtendCommand(source_gadget, target_gadget, location,
                                    keep_source)
            command_manager.execute(cmd, target_gadget.project)
            return True

        elif self._is_append_action(location):
            pos = self._get_append_position(target_gadget.widget, location)
            parent = target_gadget.get_parent()
            cmd = DragAppendCommand(source_gadget, parent, pos, keep_source)
            command_manager.execute(cmd, source_gadget.project)
            return True

        return False

    def _execute_create(self, location, adaptor, target_gadget):
        """
        Execute the create-append or create-extend command.

        @param location: the drop location
        @type location: dnd constant
        @param adaptor: adaptor for creating the class
        @type adaptor: L{gazpacho.widgetadaptor.WidgetAdaptor}
        @param target_gadget: the gadget where the drop occurred
        @type target_gadget: L{gazpacho.gadget.Gadget}

        @return: True if the drop was successful
        @rtype: bool
        """
        project = target_gadget.project
        success = False

        if self._is_extend_action(location):
            gadget = Gadget(adaptor, project)
            gadget.create_widget(interactive=True)
            cmd = CreateExtendCommand(gadget, target_gadget, location)
            command_manager.execute(cmd, project)

            success = True

        elif self._is_append_action(location):
            pos = self._get_append_position(target_gadget.widget, location)
            parent = target_gadget.get_parent()
            gadget = Gadget(adaptor, project)
            gadget.create_widget(interactive=True)
            cmd = CreateAppendCommand(gadget, parent, pos)
            command_manager.execute(cmd, project)

            success = True

        if success:
            from gazpacho.palette import palette
            if not palette.persistent_mode:
                palette.unselect_widget()

        return success

    #
    # Signal handlers for the drag source
    #

    def _on_widget__drag_begin(self, event_widget, drag_context):
        """
        Set a drag icon that matches the source widget.
        """
        source_gadget = Gadget.from_widget(event_widget).dnd_gadget
        if source_gadget:
            pixbuf = source_gadget.adaptor.icon.get_pixbuf()
            event_widget.drag_source_set_icon_pixbuf(pixbuf)

    def _on_widget__drag_data_get(self, event_widget, drag_context,
                                  selection_data, info, time):
        """Make the widget data available in the format that was
        requested.

        If the drag and drop occurs within the application the widget
        can be accessed directly otherwise it has to be passed as an
        XML string.
        """
        source_gadget = Gadget.from_widget(event_widget).dnd_gadget

        # If we can't get the widget we indicate this failure by
        # passing an empty string. Not sure if it's correct but it
        # works for us. Note that the source widget is sometimes
        # different from the dragged widget.
        data = ""
        if source_gadget:
            # The widget should be passed as XML
            if info == INFO_TYPE_XML:
                data = source_gadget.to_xml(skip_external_references=True)
            # The widget can be retrieved directly and we only pass the name
            elif info == INFO_TYPE_WIDGET:
                data = source_gadget.name

        selection_data.set(selection_data.target, 8, data)

    #
    # Signal handlers for the drag target
    #

    def _on_widget__drag_data_received(self, event_widget, drag_context,
                                       x, y, data, info, time):
        """
        The data has been received and the appropriate command can now
        be executed.

        If the drag/drop is in the same project the target handles
        everything. If it's between different projects the target only
        handles the drop (paste) and the source takes care of the drag
        (cut).
        """
        # XXX should we validate the drop-zone again? There are
        #     problems with gtk.Entry for some reason and events seem
        #     to slip through. Click on the text entry and drag onto
        #     the text entry (at the end).
        if not self._is_valid_drop_zone(drag_context, event_widget, x, y):
            # XXX this seems to solve the problem but can it be fixed
            #     in a better way? We still get the incorrect cursor.
            drag_context.finish(False, False, time)
            return
        
        # If there is no data we indicate that the drop was not successful
        success = False
        if data.data:
            target_widget, tx, ty = self._get_target_widget(event_widget, x, y)
            project = self._get_target_project(target_widget)
            location = self._get_drop_location(target_widget, tx, ty)[0]
            target_gadget = Gadget.from_widget(target_widget)

            if info in (INFO_TYPE_WIDGET, INFO_TYPE_XML):
                source_gadget = self._get_source_gadget(data, info,
                                                        drag_context, project)
                keep_source = (drag_context.action != gtk.gdk.ACTION_MOVE)
                success = self._execute_drag(location, source_gadget,
                                             target_gadget, keep_source)

            elif info == INFO_TYPE_PALETTE:
                adaptor = project.get_app().add_class
                if adaptor:
                    success = self._execute_create(location, adaptor,
                                                   target_gadget)
                else:
                    success = False

        drag_context.finish(success, False, time)

class PlaceholderDnDHandler(DnDHandler):

    def _get_target_project(self, widget):
        """
        Get the project to which the target belongs.

        @param widget: the target placeholder
        @type widget: L{gazpacho.placeholder.Placeholder}
        """
        return widget.get_project()

    def _set_drag_highlight(self, event_widget, x, y):
        """
        Highlight the widget in an appropriate way to indicate
        that this is a valid drop zone.

        @param event_widget: the target event widget
        @type event_widget: gtk.Widget
        @param x: the mouse x position
        @type x: int
        @param y: the mouse y position
        @type y: int
        """
        event_widget.drag_highlight()

    def _clear_drag_highlight(self, event_widget):
        """
        Clear the drag highligt.

        @param event_widget: the target event widget
        @type event_widget: gtk.Widget
        """
        event_widget.drag_unhighlight()

    def _execute_drag(self, source_gadget, placeholder, keep_source, project):
        """
        Execute the drag and drop command.

        @param source_gadget: the gadget that is being dragged
        @type source_gadget: L{gazpacho.gadget.Gadget}
        @param placeholder: the placeholder where the drop occurred
        @type placeholder: L{gazpacho.placeholder.Placeholder}
        @param keep_source: True if the source widget should not be removed
        @type keep_source: bool
        @param project: the target project
        @type project: L{gazpacho.project.Project}

        @return: True if the drop was successful
        @rtype: bool
        """
        if not source_gadget:
            return False

        if keep_source:
            cmd = CommandCutPaste(source_gadget.copy(project), project, placeholder,
                                  False)
        else:
            cmd = CommandDragDrop(source_gadget, placeholder)

        command_manager.execute(cmd, project)
        return True

    #
    # Signal handlers for the drag target
    #

    def _on_widget__drag_data_received(self, event_widget, drag_context,
                                       x, y, data, info, time):
        """
        The data has been received and the appropriate command can now
        be executed.

        If the drag/drop is in the same project the target handles
        everything. If it's between different projects the target only
        handles the drop (paste) and the source takes care of the drag
        (cut).
        """
        # If there is no data we indicate that the drop was not
        # successful
        if not data.data:
            drag_context.finish(False, False, time)
            return

        project = self._get_target_project(event_widget)

        success = False
        if info in (INFO_TYPE_WIDGET, INFO_TYPE_XML):
            # note that the dragged widget isn't always the same as
            # the source widget in the event
            source_gadget = self._get_source_gadget(data, info, drag_context,
                                                    project)
            keep_source = drag_context.action != gtk.gdk.ACTION_MOVE
            
            success = self._execute_drag(source_gadget, event_widget,
                                         keep_source, project)
        elif info == INFO_TYPE_PALETTE:
            gapi.create_gadget(project, project.get_app().add_class,
                               event_widget)
            success = True

        drag_context.finish(success, False, time)


class CommandDragDrop(Command):
    """Command for executing a drag and drop action. This will move
    the widget and cannot be used to drag widgets between programs.
    """

    def __init__(self, source_gadget, target_placeholder):
        """
        Initialize the command.

        @param source_gadget: the source gadget
        @type source_gadget: L{gazpacho.gadget.Gadget}
        @param target_placeholder: the target placeholder
        @type target_placeholder: L{gazpacho.placeholder.Placeholder}
        """
        description = _("Drag and Drop widget %s") % source_gadget.name
        Command.__init__(self, description)

        self._undo = False
        self._source_placeholder = None
        self._target_placeholder = target_placeholder
        self._source = source_gadget

    def execute(self):
        if self._undo:
            self._remove_gadget(self._source, self._target_placeholder)
            self._add_gadget(self._source, self._source_placeholder)

        else:
            # Delay the construction of the placeholder until we need it
            if not self._source_placeholder:
                self._source_placeholder = Placeholder()

            self._remove_gadget(self._source, self._source_placeholder)
            self._add_gadget(self._source, self._target_placeholder)

        self._undo = not self._undo

    def _remove_gadget(self, gadget, placeholder):
        """
        Remove the widget from the gadget and replace it with a
        new placeholder.

        @param gadget: the gadget that should be removed
        @type gadget: L{gazpacho.gadget.Gadget}
        @param placeholder: the new placeholder
        @type placeholder: L{gazpacho.placeholder.Placeholder}
        """
        parent = gadget.get_parent()
        Gadget.replace(gadget.widget, placeholder, parent)

        gadget.widget.hide()
        gadget.project.remove_widget(gadget.widget)

    def _add_gadget(self, gadget, placeholder):
        """
        Remove the old placeholder and add a widget from the gadget.

        @param gadget: the gadget that should be added
        @type gadget: L{gazpacho.gadget.Gadget}
        @param placeholder: the old placeholder
        @type placeholder: L{gazpacho.placeholder.Placeholder}
        """
        parent = util.get_parent(placeholder)
        project = parent.project
        Gadget.replace(placeholder, gadget.widget, parent)

        project.add_widget(gadget.widget)
        gadget.select()

        gadget.widget.show_all()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
Generic object-relational mapper (ORM) for manipulating Schevo/Durus 
databases.
"""
from operator import attrgetter

__all__ = ['RelationProxy']

class RelationProxy(object):
    """
    Allow access to a ``Extent`` object using a referenced
    string value.

        >>> objects = RelationProxy(db.User)
        >>> objects.all()
        >>> user = objects.get(username="corina")
        >>> assert str(objects) == '<RelationProxy: User>'
        >>> user2 = objects.get_by_oid(oid)
    """
    
    def __init__(self, reference):
        """Constructor method.
        params:
        ``reference``: A valid Extent class reference (str).
        """
        self.extent = reference
        self.t = getattr(reference, 't')
        self.name = str(self.extent.name)

    def all(self, **params):
        """Return all entities for a Extent."""
        if 'order_by' in params.keys():
            order_by = params.pop('order_by')
        else:
            order_by = None
        values = [item for item in self.extent.find(**params)]
        if (order_by and isinstance(order_by, str)):
            return self._order_by(values, order_by)
        else:    
            return values

    def get(self, **kwargs):
        """Get method to retrieve a single ``Entity`` object."""
        return self.extent.findone(**kwargs)
    
    def save(self, data):
        """
        Save a new record for a ``Extent`` instance (transaction wrapper)
        """
        return NotImplementedError

    def update(self, oid, data):
        """
        Update operation (Transaction wrapper)
        """
        return NotImplementedError
    
    def delete(self, oid):
        """
        Delete operation (Transaction wrapper)
        """
        return NotImplementedError
    
    def get_by_oid(self, oid):
        """
        Lookup a individual entity record using ``oid`` as the
        primary key, return None if no record was found.
        
        """
        #db = self.extent.db
        
        try:
            obj = self.extent[int(oid)]
        except KeyError:
            # no such object with such OID exists
            #raise ObjectDoesNotExist(e)
            raise
        return obj

    def _order_by(self, data, order_by):
        """Return a list of entities sorted by a field attribute."""
        values = sorted(data, key=attrgetter(order_by))
        values.reverse()
        return values
    order_by = _order_by

    def find(self, **kwargs):
        order_by = kwargs.pop('pub_date', None)
        objs = self.extent.find(**kwargs)
        if order_by:
            return self.order_by(objs, order_by)
        else:
            return objs

    def __str__(self):
        return "<RelationProxy: %s>" % self.name
    



from django.conf import settings
from django.contrib.sitemaps import Sitemap
from django.core import urlresolvers
from product.models import Category, Product
from satchmo_store.shop import get_satchmo_setting


class CategorySitemap(Sitemap):
    changefreq = 'daily'
    priority = 0.6

    def items(self):
        return Category.objects.by_site()

class ProductSitemap(Sitemap):
    changefreq = 'weekly'

    def items(self):
        return Product.objects.active_by_site(variations=False)

class MainSitemap(Sitemap):
    urls = []

    def items(self):
        return self.urls

    def add_url(self, location, priority=0.5, changefreq='weekly'):
        self.urls.append({
            'location': location,
            'priority': priority,
            'changefreq': changefreq,
        })

    def location(self, obj):
        return obj['location']

    def priority(self, obj):
        return obj['priority']

    def changefreq(self, obj):
        return obj['changefreq']

def satchmo_main():
    base = get_satchmo_setting('SHOP_BASE')
    rv = urlresolvers.reverse
    urls = (
        (base + '/', 1.0, 'hourly'),
        (rv('satchmo_contact'), 1.0, 'monthly'),
        (rv('satchmo_cart'), 0.5, 'monthly'),
        (rv('auth_login'), 0.8, 'monthly'),
        (rv('registration_register'), 0.8, 'monthly'),
        (rv('auth_password_reset'), 0.8, 'monthly'),
    )
    sitemap = MainSitemap()
    for url in urls:
        sitemap.add_url(*url)
    return sitemap

sitemaps = {
    'satchmo_main': satchmo_main,
    'category': CategorySitemap,
    'products': ProductSitemap,
}



# -*- test-case-name: gazpacho.unittest.test_uim.py -*-
# Copyright (C) 2004,2005 by SICEm S.L.
#               2005,2006 Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from xml.dom import minidom

import gtk

from gazpacho.gaction import GAction, GActionGroup

default_actions = [
    # name, label, short_label, is_important,
    # tooltip, stock_id, callback, accelerator
    ('FileMenu', '_File', None, False,
     '', None, '', ''),
    ('New', '_New', None, False,
     'Create a new file', gtk.STOCK_NEW, '', '<control>N'),
    ('Open', '_Open', None, False,
     'Open a file', gtk.STOCK_OPEN, '', '<control>O'),
    ('Save', '_Save', None, True,
      'Save a file', gtk.STOCK_SAVE, '', '<control>S'),
    ('SaveAs', 'Save _As', None, False,
     'Save with a different name', gtk.STOCK_SAVE_AS, '', ''),
    ('Quit', '_Quit', None, False,
     'Quit the program', gtk.STOCK_QUIT, '', '<control>Q'),
    ('EditMenu', '_Edit', None, False,
     '', None, '', ''),
    ('Copy', '_Copy', None, False,
      'Copy selected object into the clipboard', gtk.STOCK_COPY, '',
      '<control>C'),
    ('Cut', 'Cu_t', None, False,
      'Cut selected object into the clipboard', gtk.STOCK_CUT, '',
      '<control>X'),
    ('Paste', '_Paste', None, False,
     'Paste object from the Clipboard', gtk.STOCK_PASTE, '', '<control>V'),
    ]

# XXX: Remove
def xml_filter_nodes(nodes, node_type):
    return [node for node in nodes if node.nodeType == node_type]

def strip_text_nodes(node):
    for child in node.childNodes[:]:
        strip_text_nodes(child)

    if node.nodeType == node.TEXT_NODE:
        node.parentNode.removeChild(node)

class GazpachoUIM(object):
    """Wrapper for the UIManager.

    Provides additional functionality to the standard UIManager
    """

    def __init__(self, project):

        self._project = project

        self.reset()

    def get_name(self):
        return self._uim_name

    def reset(self):
        self._uim = gtk.UIManager()
        self._uim_name = 'uimanager'

        # this dictionary maps gadgets with their ui definitions. Example:
        # { gadget1: {'initial-state' : (XML1, merge-id1)},
        #   gadget2: {'initial-state' : (XML2, merge-id2),
        #              'special-mode' : (XML3, merge-id3)},
        # }
        self._uis = {}

        self._action_groups = []

        # this dictionary is only used in the loading phase
        self._loaded_uis = {}

    def add_ui(self, gadget, xml_string, ui_name='initial-state'):
        "Add a ui definition for gadget with the name ui_name"
        if gadget not in self._uis:
            self._uis[gadget] = {}

        # add it to the uimanager
        merge_id = self._uim.add_ui_from_string(xml_string)
        self._uis[gadget][ui_name] = (xml_string, merge_id)

    def get_ui(self, gadget, ui_name=None):
        "Return a ui definition for gadget or all of them if ui_name is None"
        uis = self._uis.get(gadget)
        if ui_name and uis:
            return uis[ui_name]
        return uis

    def update_ui(self, gadget, xml, ui_name='initial-state',
                  refresh_widget=True):
        """Update the ui definition of gadget with name ui_name

        If the argument refresh_widget is True the gtk widget associated
        with gadget is created again with the new ui and the gadget is
        updated with this new widget
        """

        project = gadget.project
        if refresh_widget:
            # save the parent and the packing props before changing the uim
            old_widget = gadget.widget
            parent = old_widget.get_parent()
            if parent:
                props = {}
                ptype = type(parent)
                for pspec in gtk.container_class_list_child_properties(ptype):
                    props[pspec.name] = parent.child_get_property(old_widget,
                                                                  pspec.name)

                parent.remove(old_widget)

                # Update project. We have to check if the widget is still in
                # the project to play nice with widget name changes
                if project.get_gadget_by_name(old_widget.get_name()):
                    project.remove_widget(old_widget)

        # update the ui manager
        old_merge_id = self._uis[gadget][ui_name][1]
        self._uim.remove_ui(old_merge_id)
        self._uim.ensure_update()
        new_merge_id = self._uim.add_ui_from_string(xml)
        self._uis[gadget][ui_name] = (xml, new_merge_id)

        if refresh_widget:
            # update the gtk widget of gadget
            new_widget = self.get_widget(gadget)

            # copy the packing properties to the new widget
            if parent:
                parent.add(new_widget)

                for name, value in list(props.items()):
                    parent.child_set_property(new_widget, name, value)

            # finish widget configuration
            gadget.setup_widget(new_widget)

            # Update project
            project.add_widget(new_widget)

    def get_widget(self, gadget):
        self._uim.ensure_update()
        return self._uim.get_widget('/%s' % gadget.name)

    def add_action_group(self, action_group):
        if self._uim is None:
            self._uim = gtk.UIManager()

        self._action_groups.append(action_group)
        id1 = action_group.connect('add-action', self._project._add_action_cb)
        id2 = action_group.connect('remove-action',
                                   self._project._remove_action_cb)
        action_group.set_data('add_action_id', id1)
        action_group.set_data('remove_action_id', id2)

        action_group.create_gtk_action_group(self._uim)

    def remove_action_group(self, action_group):
        self._action_groups.remove(action_group)
        id1 = action_group.get_data('add_action_id')
        id2 = action_group.get_data('remove_action_id')
        action_group.disconnect(id1)
        action_group.disconnect(id2)

        action_group.destroy_gtk_action_group()

    def get_action_group(self, action_group_name):
        """Return an action group given its name or None if no action group
        with that name was found"""
        for gag in self._action_groups:
            if gag.get_name() == action_group_name:
                return gag

    def create_default_actions(self):
        """Create a set of default action groups inside a DefaultActions
        action group in the project this uim belongs to.
        Don't add them if they are already there.
        """
        # create the 'DefaultActions' action group
        gaction_group = None
        for gag in self._action_groups:
            if gag.name == 'DefaultActions':
                gaction_group = gag
                break
        # if not found, create it
        else:
            gag = GActionGroup('DefaultActions')
            self._project.add_action_group(gag)
            gaction_group = self._action_groups[-1]

        gaction_group.add_actions(default_actions)

    def update_widget_name(self, gadget):
        """Update the ui definitions of gadget with the new widget name"""
        if self._uim is None:
            raise RuntimeError("No UIManager has been created yet. Can not "
                               "update the name of the widget %s" % \
                               gadget.name)

        for ui_name, value in list(self._uis[gadget].items()):
            new_xml = self._change_ui_definition(value[0], gadget.name)
            self.update_ui(gadget, new_xml, ui_name)

    def _change_ui_definition(self, xml_string, new_name):
        "Change a ui definition of a widget to use a new_name for the widget"
        doc = minidom.parseString(xml_string)
        doc.documentElement.setAttribute('action', new_name)
        doc.documentElement.setAttribute('name', new_name)
        return doc.documentElement.toxml()

    def load(self, loader):
        """Get the UIManager of the loader and copy its content into
        this uimanager.

        Load also its ui definition spliting them into the widgets. When
        this uim is loaded the widgets it defines (menubars, toolbars, ...)
        are not loaded yet so we can't add the ui definitions into the
        self.uis dictionary. We use a temporary one until we load the
        widgets."""

        objs = [obj for obj in loader.get_widgets()
                        if isinstance(obj, gtk.UIManager)]
        if not objs:
            return

        if len(objs) >= 2:
            print('Warning: multiple uimanagers is not supported yet')

        gaction_callbacks = {}
        uimanager = objs[0]
        self._uim_name = uimanager.get_data('gazpacho::object-id')

        for signal_info in loader.get_signals():
            widget, signal, handler = signal_info[:3]
            if signal == "activate" and isinstance(widget, gtk.Action):
                gaction_callbacks[widget] = handler

        # read the action groups
        for ag in uimanager.get_action_groups():
            # only add the action group if we don't have it
            gag = self.get_action_group(ag.get_name())
            if not gag:
                gag = GActionGroup.new(ag)
                self.add_action_group(gag)
            for action in ag.list_actions():
                ga = GAction.new(action, gag)
                # only add the action if the action group does not have it
                if not gag.get_action(ga.name):
                    ga.callback = gaction_callbacks.get(action, '')
                    gag.add_action(ga)

        # load the ui definitions
        for ui_name, xml_string, merge_id in loader.get_ui_definitions():
            # we need to split this ui definition into several widgets
            doc = minidom.parseString(xml_string)
            self._split_ui(doc, ui_name, merge_id)

    def _split_ui(self, document, ui_name, merge_id):
        """Split a XML tree into several ones, each one with only a widget
        definition"""
        root = document.documentElement
        for node in xml_filter_nodes(root.childNodes, root.ELEMENT_NODE):
            xml_string = node.toxml()
            name = node.getAttribute('name')
            if name not in self._loaded_uis:
                self._loaded_uis[name] = []

            self._loaded_uis[name].append((ui_name, xml_string, merge_id))

    def load_widget(self, gadget, old_name):
        """Load the gadget replacing its widget by the one the uimanager
        has.

        old_name is the name the widget had before creating the gadget from
        it and we are inside a paste operation it will be different to the
        name the gadget has now.

        Also moves the dictionary self.loaded_uis into the self.uis"""

        if old_name not in self._loaded_uis:
            return

        self._uis[gadget] = {}
        for ui_name, xml_string, merge_id in self._loaded_uis[old_name]:
            if gadget.name != old_name:
                # we need to change the xml_string to change the widget name
                xml_string = self._change_ui_definition(xml_string,
                                                        gadget.name)

            merge_id = self._uim.add_ui_from_string(xml_string)
            self._uis[gadget][ui_name] = (xml_string, merge_id)

        del self._loaded_uis[old_name]

    def is_loading(self):
        return len(list(self._loaded_uis.keys())) > 0

    def remove_gadget(self, gadget):
        "Remove all the uidefinitions of gadget on the uimanager"
        for value in list(self._uis[gadget].values()):
            self._uim.remove_ui(value[1])

        del self._uis[gadget]

    def add_gadget(self, gadget, definitions):
        """Add an existing widget with its definitions to the uim.

        Also, update the widget of gadget after this change
        """
        self._uis[gadget] = {}
        for ui_name, value in list(definitions.items()):
            xml_string = value[0]
            merge_id = self._uim.add_ui_from_string(xml_string)
            self._uis[gadget][ui_name] = (xml_string, merge_id)

        widget = self.get_widget(gadget)
        gadget.setup_widget(widget)

    def save_ui_definitions(self, document, gadgets, version):
        """Return a list of XML nodes with the ui definitions of the
        gadgets we need to save
        """
        uis = self._group_ui_definitions_by_name(gadgets)
        uis = self._get_required_uis(uis, gadgets)

        nodes = []
        for ui_name, xml_list in list(uis.items()):
            ui_node = document.createElement('ui')
            ui_node.setAttribute('id', ui_name)

            uidef_node = document.createElement('ui')
            text_node = minidom.Text()
            text_node.data = '\n'
            uidef_node.appendChild(text_node)
            for xml_string in xml_list:
                tmp = minidom.parseString(xml_string)
                uidef_node.appendChild(tmp.documentElement)
            text_node = minidom.Text()
            text_node.data = '\n'
            uidef_node.appendChild(text_node)

            if version == 'gtkbuilder':
                strip_text_nodes(uidef_node)
                nodes.append(uidef_node)
            else:
                child = document.createCDATASection(uidef_node.toxml())
                ui_node.appendChild(child)
                nodes.append(ui_node)

        return nodes

    def get_ui_widgets(self, widget):
        """Get a list of all the widgets inside the hierarchy that are widgets
        related to a UI Manager (e.g. Toolbars and Menubars).
        """
        result = []
        if isinstance(widget, (gtk.Toolbar, gtk.MenuBar)):
            result.append(widget)

        elif isinstance(widget, gtk.Container):
            for child in widget.get_children():
                result += self.get_ui_widgets(child)

        return result

    def _group_ui_definitions_by_name(self, gadgets):
        """Group the ui definitions of this UI Manager by name.

        Only group those that belong to any of gadgets
        In the UIManager they are usually stored by widget.
        """
        uis = {}
        keys = list(self._uis.keys())
        keys.sort(lambda a, b: cmp(a.name, b.name))
        for gadget in keys:
            if gadget not in gadgets:
                continue
            ui_def = self._uis[gadget]
            for ui_name, value in list(ui_def.items()):
                xml_string = value[0]
                uis.setdefault(ui_name, []).append(xml_string)
        return uis

    def _get_required_uis(self, ui_definitions, gadgets):
        """Return the ui definitions that the gadgets are using"""
        # In this dict we decide if we need to save this ui or not
        # Initialy we don't save any ui
        uis_to_save = dict([(k, False) for k in list(ui_definitions.keys())])

        # now, let's loop through the widgets we need to save to see
        # if we need to save their uis
        for gadget in gadgets:
            gadget_uis = self.get_ui(gadget)
            if not gadget_uis:
                continue

            uis_names = list(gadget_uis.keys())
            for ui_name in uis_names:
                if ui_name in list(uis_to_save.keys()):
                    uis_to_save[ui_name] = True

        # so let's save the ui definitions
        for ui_name, save in list(uis_to_save.items()):
            if not save:
                del ui_definitions[ui_name]

        return ui_definitions

    def get_action_groups(self):
        return self._action_groups



from django import forms
from django.utils.translation import ugettext_lazy as _
from payment.forms import SimplePayShipForm

class GiftCertCodeForm(forms.Form):
    code = forms.CharField(_('Code'), required=True)
    
class GiftCertPayShipForm(SimplePayShipForm):
    giftcode = forms.CharField(max_length=100)

    def __init__(self, request, paymentmodule, *args, **kwargs):
        super(GiftCertPayShipForm, self).__init__(request, paymentmodule, *args, **kwargs)



from decimal import Decimal
from django.contrib.sites.models import Site
from livesettings import config_value
from l10n.utils import moneyfmt
from product.models import ProductVariation, Option, split_option_unique_id, \
                                   ProductPriceLookup, OptionGroup, Discount, \
                                   NullDiscount, Product
from satchmo_utils.numbers import RoundedDecimalError, round_decimal
import datetime
import logging
import types
import string

log = logging.getLogger('product.utils')

def calc_discounted_by_percentage(price, percentage):
    if not percentage:
        return price
    if percentage > 1:
        log.warn("Correcting discount percentage, should be less than 1, is %s", percentage)
        percentage = percentage/100

    work = price * (1-percentage)
    cents = Decimal("0.01")
    return work.quantize(cents)

def find_auto_discounts(product):
    if not type(product) in (types.ListType, types.TupleType):
        product = (product,)
    today = datetime.date.today()
    discs = Discount.objects.filter(automatic=True, active=True, startDate__lte=today, endDate__gt=today)
    return discs.filter(validProducts__in=product).order_by('-percentage')

def find_best_auto_discount(product):
    discs = find_auto_discounts(product)
    if len(discs) > 0:
        return discs[0]
    else:
        return None

def productvariation_details(product, include_tax, user, create=False):
    """Build the product variation details, for conversion to javascript.

    Returns variation detail dictionary built like so:
    details = {
        "OPTION_KEY" : {
            "SLUG": "Variation Slug",
            "PRICE" : {"qty" : "$price", [...]},
            "SALE" : {"qty" : "$price", [...]},
            "TAXED" : "$taxed price",   # omitted if no taxed price requested
            "QTY" : 1
        },
        [...]
    }
    """

    ignore_stock = config_value('PRODUCT','NO_STOCK_CHECKOUT')
    discount = find_best_auto_discount(product)
    use_discount = discount and discount.percentage > 0

    if include_tax:
        from tax.utils import get_tax_processor
        taxer = get_tax_processor(user=user)
        tax_class = product.taxClass

    details = {'SALE' : use_discount}
    
    variations = ProductPriceLookup.objects.filter(parentid=product.id).order_by("-price")
    if variations.count() == 0:
        if create:
            log.debug('Creating price lookup for %s', product)
            ProductPriceLookup.objects.smart_create_for_product(product)
            variations = ProductPriceLookup.objects.filter(parentid=product.id).order_by("-price")
        else:
            log.warning('You must run satchmo_rebuild_pricing and add it to a cron-job to run every day, or else the product details will not work for product detail pages.')
    for detl in variations:
        key = detl.key
        if details.has_key(key):
            detail = details[key]
            qty = detl.quantity
        else:
            detail = {}
            detail['SLUG'] = detl.productslug

            if not detl.active:
                qty = round_decimal('-1.0')
            elif ignore_stock:
                qty = round_decimal('10000.0')
            else:
                qty = round_decimal(detl.items_in_stock)

            detail['QTY'] = round_decimal(qty)

            detail['PRICE'] = {}
            
            if use_discount:
                detail['SALE'] = {}
                
            if include_tax:
                detail['TAXED'] = {}
                if use_discount:
                    detail['TAXED_SALE'] = {}
                
            details[key] = detail

        qtykey = "%d" % detl.quantity
        
        price = detl.dynamic_price
        
        detail['PRICE'][qtykey] = moneyfmt(price)
        if use_discount:
            detail['SALE'][qtykey] = moneyfmt(calc_discounted_by_percentage(price, discount.percentage))
        
        if include_tax:
            tax_price = taxer.by_price(tax_class, price) + price
            detail['TAXED'][qtykey] = moneyfmt(tax_price)
            if use_discount:
                detail['TAXED_SALE'][qtykey] = moneyfmt(calc_discounted_by_percentage(tax_price, discount.percentage))
                
    return details
    
def rebuild_pricing():
    site = Site.objects.get_current()
    for lookup in ProductPriceLookup.objects.filter(siteid=site.id):
        lookup.delete()
    
    products = Product.objects.active_by_site(site=site, variations=False)
    
    productct = products.count()
    pricect = 0
    
    for product in products:        
        prices = ProductPriceLookup.objects.smart_create_for_product(product)
        pricect += len(prices)
    
    return productct, pricect

def serialize_options(product, selected_options=()):
    """
    Return a list of optiongroups and options for display to the customer.
    Only returns options that are actually used by members of this product.

    Return Value:
    [
    {
    name: 'group name',
    id: 'group id',
    items: [{
        name: 'opt name',
        value: 'opt value',
        price_change: 'opt price',
        selected: False,
        },{..}]
    },
    {..}
    ]

    Note: This doesn't handle the case where you have multiple options and
    some combinations aren't available. For example, you have option_groups
    color and size, and you have a yellow/large, a yellow/small, and a
    white/small, but you have no white/large - the customer will still see
    the options white and large.
    """    
    all_options = product.get_valid_options()
    group_sortmap = OptionGroup.objects.get_sortmap()

    # first get all objects
    # right now we only have a list of option.unique_ids, and there are
    # probably a lot of dupes, so first list them uniquely
    values = []
    
    if all_options != [[]]:
        vals = {}
        groups = {}
        opts = {}
        serialized = {}
        
        for options in all_options:
            for option in options:
                if not opts.has_key(option):
                    k, v = split_option_unique_id(option)
                    vals[v] = False
                    groups[k] = False
                    opts[option] = None
        
        for option in Option.objects.filter(option_group__id__in = groups.keys(), value__in = vals.keys()):
            uid = option.unique_id
            if opts.has_key(uid):
                opts[uid] = option

        # now we have all the objects in our "opts" dictionary, so build the serialization dict

        for option in opts.values():
            if not serialized.has_key(option.option_group_id):
                serialized[option.option_group.id] = {
                    'name': option.option_group.translated_name(),
                    'description': option.option_group.translated_description(),
                    'id': option.option_group.id,
                    'items': [],
                }
            if not option in serialized[option.option_group_id]['items']:
                serialized[option.option_group_id]['items'] += [option]
                option.selected = option.unique_id in selected_options

        # first sort the option groups
        for k, v in serialized.items():
            values.append((group_sortmap[k], v))
        
        if values:
            values.sort()
            values = zip(*values)[1]

        #now go back and make sure option items are sorted properly.
        for v in values:
            v['items'] = _sort_options(v['items'])
    
    log.debug('serialized: %s', values)
    
    log.debug('Serialized Options %s: %s', product.product.slug, values)
    return values

def _sort_options(lst):
    work = [(opt.sort_order, opt) for opt in lst] 
    work.sort()
    return zip(*work)[1]

# All the functions below are used to validate custom attributes
# associated with a product.
# Custom ones can be added to the list via the admin setting ATTRIBUTE_VALIDATION
    
def validation_simple(value, product=None):
    """
    Validates that at least one character has been entered.
    Not change is made to the value.
    """
    if len(value) > 1:
        return True, value
    else:
        return False, value

def validation_integer(value, product=None):
    """
   Validates that value is an integer number.
   No change is made to the value
    """
    try:
        check = int(value)
        return True, value
    except:
        return False, value

def validation_yesno(value, product=None):
    """
    Validates that yes or no is entered. 
    Converts the yes or no to capitalized version
    """
    if string.upper(value) in ["YES","NO"]:
        return True, string.capitalize(value)
    else:
        return False, value
        
def validation_decimal(value, product=None):
    """
    Validates that the number can be converted to a decimal
    """
    try:
        check = Decimal(value)
        return True, value
    except:
        return False, value



####################################################################
# Last step in the order process - confirm the info and process it
#####################################################################

from django.core import urlresolvers
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from livesettings import config_value
from satchmo_store.shop.models import Order, OrderStatus
from payment.config import gateway_live
from satchmo_utils.dynamic import lookup_url, lookup_template
from satchmo_store.shop.models import Cart
from payment import signals
from payment.modules.base import ProcessorResult
import logging

log = logging.getLogger('payment.views')

class ConfirmController(object):
    """Centralizes and manages data used by the confirm views.
    Generally, this is used by initializing, then calling
    `confirm`.  If defaults need to be overridden, such as
    by setting different templates, or by overriding `viewTax`,
    then do that before calling `confirm`.
    """

    def __init__(self, request, payment_module, extra_context={}):
        self.request = request
        self.paymentModule = payment_module
        if payment_module:
            processor_module = payment_module.MODULE.load_module('processor')
            self.processor = processor_module.PaymentProcessor(self.paymentModule)
        else:
            self.processor = None
        self.viewTax = config_value('TAX', 'DEFAULT_VIEW_TAX')
        self.order = None
        self.cart = None
        self.extra_context = extra_context
                
        #to override the form_handler, set this
        #otherwise it will use the built-in `_onForm`
        self.onForm = self._onForm
        
        #to override the success method, set this
        #othewise it will use the built-in `_onSuccess`
        self.onSuccess = self._onSuccess
        
        #false on any "can not continue" error
        self.valid = False
        
        #the value to be returned from the view
        #an HttpResponse or a HttpRedirect
        self.response = None
        
        self.processorMessage = ""
        self.processorReasonCode = ""
        self.processorResults = None
        
        self.templates = {
            'CONFIRM' : 'shop/checkout/confirm.html',
            'EMPTY_CART': 'shop/checkout/empty_cart.html',
            '404': 'shop/404.html',
            }
            
                
    def confirm(self, force_post=False):
        """Handles confirming an order and processing the charges.

        If this is a POST, then tries to charge the order using the `payment_module`.`processor`
        On success, sets `response` to the result of the `success_handler`, returns True
        On failure, sets `response` to the result, the result of the `form_handler`, returns False
        
        If not a POST, sets `response` to the result, the result of the `form_handler`, returns True
        """
        if not self.sanity_check():
            return False

        status = False

        if force_post or self.request.method == "POST":
            self.processor.prepare_data(self.order)
            # This copy command is used to handle an error that can occur
            # with mod_wsgi. See #951 for more info
            tmp = self.request.POST.copy()
            if self.process():
                self.response = self.onSuccess(self)
                return True
                
        else:
            # not a post, so still a success
            status = True

        self.response = self.onForm(self)
        return status
        
        
    def invalidate(self, dest):
        """Mark the confirmation invalid, and set the response"""
        self.valid = False
        self.response = dest


    def lookup_template(self, key):
        """Shortcut method to the the proper template from the `paymentModule`"""
        return lookup_template(self.paymentModule, self.templates[key])


    def lookup_url(self, view):
        """Shortcut method to the the proper url from the `paymentModule`"""
        return lookup_url(self.paymentModule, view)

        
    def _onForm(self, controller):
        """Show the confirmation page for the order.  Looks up the proper template for the
        payment_module.
        """
        template = controller.lookup_template('CONFIRM')
        controller.order.recalculate_total()
        
        base_env = {
            'PAYMENT_LIVE' : gateway_live(controller.paymentModule),
            'default_view_tax' : controller.viewTax,
            'order': controller.order,
            'errors': controller.processorMessage,
            'checkout_step2': controller.lookup_url('satchmo_checkout-step2')}
        if controller.extra_context:
            base_env.update(controller.extra_context)
            
        context = RequestContext(self.request, base_env)
        return render_to_response(template, context_instance=context)

    def _onSuccess(self, controller):
        """Handles a success in payment.  If the order is paid-off, sends success, else return page to pay remaining."""
        if controller.order.paid_in_full:
            controller.cart.empty()
            for item in controller.order.orderitem_set.all():
                if item.product.is_subscription:
                    item.completed = True
                    item.save()
            try:
                curr_status = controller.order.orderstatus_set.latest()  
            except OrderStatus.DoesNotExist:
                curr_status = None
                
            if (curr_status is None) or (curr_status.notes and curr_status.status == "New"):
                controller.order.add_status(status='New', notes = "Order successfully submitted")
            else:
                # otherwise just update and save
                if not curr_status.notes:
                    curr_status.notes = _("Order successfully submitted")
                curr_status.save()                

            #Redirect to the success page
            url = controller.lookup_url('satchmo_checkout-success')
            return HttpResponseRedirect(url)    

        else:
            log.debug('Order #%i not paid in full, sending to pay rest of balance', controller.order.id)
            url = controller.order.get_balance_remaining_url()
            return HttpResponseRedirect(url)

    def process(self):
        """Process a prepared payment"""
        result = self.processor.process()
        self.processorResults = result.success
        if result.payment:
            reason_code = result.payment.reason_code
        else:
            reason_code = ""
        self.processorReasonCode = reason_code
        self.processorMessage = result.message

        log.info("""Processing %s transaction with %s
        Order %i
        Results=%s
        Response=%s
        Reason=%s""", self.paymentModule.LABEL.value, self.paymentModule.KEY.value, 
                      self.order.id, self.processorResults, self.processorReasonCode, self.processorMessage)
        return self.processorResults

    def sanity_check(self):
        """Ensure we have a valid cart and order."""
        try:
            self.order = Order.objects.from_request(self.request)
            
        except Order.DoesNotExist:
            url = urlresolvers.reverse('satchmo_checkout-step1')
            self.invalidate(HttpResponseRedirect(url))
            return False

        try:
            self.cart = Cart.objects.from_request(self.request)
            if self.cart.numItems == 0 and not self.order.is_partially_paid:
                template = self.lookup_template('EMPTY_CART')
                self.invalidate(render_to_response(template,
                                                   context_instance=RequestContext(self.request)))
                return False
                
        except Cart.DoesNotExist:
            template = self.lookup_template('EMPTY_CART')
            self.invalidate(render_to_response(template,
                                               context_instance=RequestContext(self.request)))
            return False

        # Check if the order is still valid
        if not self.order.validate(self.request):
            context = RequestContext(self.request, 
                {'message': _('Your order is no longer valid.')})
            self.invalidate(render_to_response(self.templates['404'],
                                               context_instance=context))

        self.valid = True
        signals.confirm_sanity_check.send(self, controller=self)
        return True

def credit_confirm_info(request, payment_module, template=None):
    """A view which shows and requires credit card selection.  
    This is the simplest confirmation flow, with no overrides."""

    controller = ConfirmController(request, payment_module)
    if template:
        controller.templates['CONFIRM'] = template
    controller.confirm()
    return controller.response
credit_confirm_info = never_cache(credit_confirm_info)

class FakeValue(object):
    def __init__(self, val):
        self.value = val

class FreeProcessor(object):

    def __init__(self, key):
        self.KEY = FakeValue(key)
        self.LABEL = FakeValue('Free Processor')

    def has_key(*args):
        return False

        
class FreeProcessorModule(object):
    def __init__(self, key):
        self.KEY = FakeValue(key)
        self.LABEL = FakeValue('Free Processor Module')

    def prepare_data(self, order, *args, **kwargs):
        self.order = order

    def process(self, *args, **kwargs):
        if self.order.paid_in_full:
            return ProcessorResult('FREE', True, _('Success'))
        else:
            return ProcessorResult('FREE', False, _('This order does not have a zero balance'))
    

def confirm_free_order(request, key="FREE", template=None):
    controller = ConfirmController(request, None)
    freeproc = FreeProcessor(key)
    freemod = FreeProcessorModule(key)
    controller.paymentModule = freeproc
    controller.processor = freemod
    if template:
        controller.templates['CONFIRM'] = template
    controller.confirm(force_post=True)
    return controller.response



# <LICENSE=ISC>

"""Helper functions mostly useful for web applications testing.
"""
import sys, os

from .collection2 import TestCollection, unittest

__all__ = ('SimpleTestRunner', )

class SimpleTestRunner(unittest.TextTestRunner):
    """
    A simplified ``unittest.TextTestRunner`` subclass with 
    RFC-822 style configuration.
    """
    
    #config_options = ('collections', 'exclude', 'quiet', 'extra_libs')
    defaults = {'quiet' : True}

    def __init__(self, namespace, verbosity=1, 
        keep_going=False, interactive=False):
        """Setup default options for the TestRunner object. 
        
        XXX: Put more documentation here.
        """
        self.quiet = self.defaults['quiet']

        if int(verbosity) >= 2:
            self.quiet = False

        self.app_conf = namespace.app_conf
        self.collections = namespace.collections
        self.vpath = namespace.vpath
        self.verbosity = int(verbosity)
        self.keep_going = keep_going
        self.interactive = bool(interactive)

        # add any extra paths to sys.path
        if self.vpath is not None:
            sys.path.extend([dirname for dirname in self.vpath if \
                os.path.exists(dirname) ])


    def __call__(self, **kwargs):
        """Invokes the ``run`` method. """
        return self.run(**kwargs)

    def run(self, **kwargs):
        """Run the test-suite using ``TextTestRunner``"""

        loader = TestCollection(quiet=self.quiet, debug=self.verbosity)
        suite = loader.loadTestsFromNames(self.collections, 
            keep_going=self.keep_going,
            interactive=self.interactive
            )
        unittest.TextTestRunner(verbosity=self.verbosity).run(suite)




from django.contrib import admin
from django.forms import models, ValidationError
from django.utils.translation import get_language, ugettext_lazy as _
from models import TaxBoundry, TaxRate, Taxable

ADDRESS_FIELDS = (
    'oddEven', 'lowAddress', 'highAddress',
    'streetPreDirection', 'streetName', 'streetSuffix', 'streetPostDirection',
    'addressSecondaryAbbr', 'addressSecondaryOddEven',
    'addressSecondaryLow', 'addressSecondaryHigh',
    'cityName', 'zipCode', 'plus4',
)

ZIP_FIELDS = ('zipCodeLow','zipExtensionLow','zipCodeHigh','zipExtensionHigh')

RATE_FIELDS = (
    'serCode',
    'fipsStateCode',
    'fipsStateIndicator',
    'fipsCountyCode',
    'fipsPlaceCode',
    'fipsPlaceType',
    'special_1_code',
    'special_1_type',
    'special_2_code',
    'special_2_type',
    'special_3_code',
    'special_3_type',
    'special_4_code',
    'special_4_type',
    'special_5_code',
    'special_5_type',
    'special_6_code',
    'special_6_type',
    'special_7_code',
    'special_7_type',
    'special_8_code',
    'special_8_type',
    'special_9_code',
    'special_9_type',
    'special_10_code',
    'special_10_type',
    'special_11_code',
    'special_11_type',
    'special_12_code',
    'special_12_type',
    'special_13_code',
    'special_13_type',
    'special_14_code',
    'special_14_type',
    'special_15_code',
    'special_15_type',
    'special_16_code',
    'special_16_type',
    'special_17_code',
    'special_17_type',
    'special_18_code',
    'special_18_type',
    'special_19_code',
    'special_19_type',
    'special_20_code',
    'special_20_type', 
)

class TaxBoundryOptions(admin.ModelAdmin):
    fieldsets = (
        (None, {
            'fields': ('recordType', 'startDate', 'endDate')
        }),
        ('Zip and +4 Records',{
            'classes': ('collapse',),
            'fields': ZIP_FIELDS,
        }),
        ('Address Records', {
            'classes': ('collapse',),
            'fields': ADDRESS_FIELDS,
        }),
        ('Rates', {
            'fields': RATE_FIELDS,
        })
    )
    list_display = ('recordType', 'startDate', 'endDate', 'zip_range', 'streetName', 'cityName', 'zipCode',)

class TaxBoundryForm(models.ModelForm):

    def clean(self):
        type = self.cleaned_data['recordType']
        if type in ('4', 'Z'):
            for field in ADDRESS_FIELDS:
                self.cleaned_data[field] = None
        if type == 'Z':
            self.cleaned_data['zipExtensionLow'] = None
            self.cleaned_data['zipExtensionHigh'] = None
            low = self.cleaned_data['zipCodeLow']
            high = self.cleaned_data['zipCodeHigh']
            if high is None or low is None:
                raise ValidationError(_("Zip-5 records need a high and a low."))
        elif type == '4':
            low = self.cleaned_data['zipCodeLow']
            high = self.cleaned_data['zipCodeHigh']
            low_ext = self.cleaned_data['zipExtensionLow']
            high_ext = self.cleaned_data['zipExtensionHigh']
            if high is None or low is None or low_ext is None or high_ext is None:
                raise ValidationError(_("Zip+4 records need a high and a low for both parts."))
        else:
            for field in ZIP_FIELDS:
                self.cleaned_data[field] = None
            for field in ('lowAddress', 'highAddress', 'streetName', 'cityName',
                          'zipCode', 'plus4',):
                if not self.cleaned_data[field]:
                    raise ValidationError(_('Address rocord needs: low, high, street, city, zip, zip+4'))
        return super(TaxBoundryForm, self).clean()

class TaxRateOptions(admin.ModelAdmin):
    list_display = ('state',
                    'jurisdictionType', 'jurisdictionFipsCode',
                    'startDate', 'endDate',
                    'generalRateIntrastate', 'generalRateInterstate',
                    'foodRateIntrastate', 'foodRateInterstate'
    )

#class TaxRateOptions(admin.ModelAdmin):
#
#    jurisdictionType = models.CharField(max_length=2,
#        verbose_name=_('Jurisdiction Type'))
#    jurisdictionFipsCode = models.CharField(max_length=5,
#        verbose_name=_('Jurisdiction FIPS Code'))
#    generalRateIntrastate = models.DecimalField(max_digits=8, decimal_places=7,
#        verbose_name=_('General Tax Rate - Intrastate'))
#    generalRateInterstate = models.DecimalField(max_digits=8, decimal_places=7,
#        verbose_name=_('General Tax Rate - Interstate'))
#    foodRateIntrastate = models.DecimalField(max_digits=8, decimal_places=7,
#        verbose_name=_('Food/Drug Tax Rate - Intrastate'))
#    foodRateInterstate = models.DecimalField(max_digits=8, decimal_places=7,
#        verbose_name=_('Food/Drug Tax Rate - Interstate'))
#    startDate = models.DateField(verbose_name=_('Effective Start Date'))
#    endDate = models.DateField(verbose_name=_('Effective End Date'))
##
#    def clean(self):
#        tax_zone = self.cleaned_data['taxZone']
#        tax_country = self.cleaned_data['taxCountry']
#        if tax_zone and not tax_country or not tax_zone and tax_country:
#            return super(TaxRateForm, self).clean()
#        else:
#            raise ValidationError(_("You must choose a zone or a country."))
#
#class TaxRateOptions(admin.ModelAdmin):
#    list_display = ("taxClass", "taxZone", "taxCountry", "display_percentage")
#    form = TaxRateForm
#

admin.site.register(TaxBoundry, TaxBoundryOptions)
admin.site.register(TaxRate, TaxRateOptions)
admin.site.register(Taxable)
#admin.site.register(TaxCollected)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2007-2013 Etienne Robillard <erob@gthcfoundation.org>
# All rights reserved.
#
# <LICENSE=ISC> 
"""
Security views for authentication and authorization of registered users based on their
role and permissions. Custom permissions and roles may be configured using the
Authkit library.
"""

import time
import hashlib
import logging
import forms

#from pubsub import pub
#import beaker.session
from notmm.utils.configparse import loadconf
from notmm.utils.wsgilib     import HTTPRedirectResponse
from notmm.controllers.auth  import AuthCookieController

from notmm.utils.template    import direct_to_template
from authkit.authenticate   import valid_password
from authkit.permissions    import RemoteUser
from authkit.authorize      import NotAuthenticatedError

from test_support import (
    WSGIControllerTestCase, 
    TestClient, 
    unittest,
    make_app,
    settings
    )

log = logging.getLogger('notmm.controllers.wsgi')
auth_conf = loadconf('development.ini', section='authkit')


class LoginControllerTestCase(WSGIControllerTestCase):
    
    def test_authenticate_user(self): 
        def authenticate_user(request, username, password, tokens='', 
            user_data=time.ctime,
            authfunc='authkit.set_user'):
            #"""Authenticate the user into the site and update the last_modified
            #timestamp if authentication and authorization granted user access."""

            # Add the cookie middleware
            # secret = hashlib.md5(username).hexdigest()
            # request = AuthTKTMiddleware(request, secret)
            request = AuthCookieController(request, auth_conf=auth_conf, settings=None)
            try:
                user_setter_func = request.environ[authfunc]
                if valid_password(request.environ, username, password):
                    user_setter_func(username, tokens=tokens, user_data=user_data())
                    #trigger function here to update the last_modified timestamp 
                    log.debug('User %s has been authenticated and authorized access!!' % username)
                raise NotAuthenticatedError
            except (KeyError, Exception):
                raise NotAuthenticatedError
            return None
    
def logout(request, template_name='auth/logout.html',
    logout_func='authkit.logout_user', user_func='authkit.set_user', 
    session_key='beaker.session', urlto='/'):
    for key in ('REMOTE_USER', 'USER'):
        if key in request.environ.keys():
            del request.environ[key]
            log.debug('logout: deleted %s' % str(key))
    
    # sanity checks
    rv = True
    for key in (logout_func,):
        if key in request.environ:
            rv = request.environ[key]()
            assert rv == None, 'fatal error deleting session!'

    # Delete beaker cache session
    if session_key in request.environ:
        Session = request.environ[session_key]
        Session.delete()
        log.debug('Beaker session deleted!')

    # Delete paste artefacts
    if 'paste.cookies' in request.environ:
        request.environ['paste.cookies'] = []
        log.debug('Paste cookies deleted!')

    return HTTPRedirectResponse(urlto)


def login(request, template_name='auth/login.mako', redirect_field_name='next',
    login_form=None, ssl=False):
    """Main login view for authentication and authorization of remote users
    using cookie-based middleware and secret key. 

    See ``notmm.controllers.auth.AuthkitController`` for more details.
    """
    path_info = request.environ['PATH_INFO']
    redirect_to = request.GET.get(redirect_field_name, path_info)
    server_name = request.environ['SERVER_NAME']
    server_port = request.environ['SERVER_PORT']
    netloc = server_name + ":" + server_port + redirect_to
    redirect_to = 'http://' + netloc
    data = dict(login_form=forms.LoginForm(), action=path_info, msg=None)
    if request.method == 'POST':
        # authenticate/authorize the user if POST
        #assert isinstance(request.POST, dict), 'POST data not properly serialized!'

        form = forms.LoginForm(request.POST.copy())
        if form.is_valid():
            # basic form validation was ok
            username = form.cleaned_data['username']
            password = form.cleaned_data['password']

            try:
                authenticate_user(request, username, password) 
            except NotAuthenticatedError:
                data['msg'] = 'Username or password incorrect. Please try again.'
            else:
                return HTTPRedirectResponse(redirect_to)
    
    # User is already authenticated or is using GET to access the 
    # login screen.
    return direct_to_template(request, template_name, extra_context=data)

def unauthorized(request):
    '''Denies access middleware to unauthorized users'''
    # Only registered accounts may create blog entries
    from notmm.utils.wsgilib import HTTPUnauthorized
    message = '''\
<html>
<head>
 <title>Permission denied</title>
</head>
<body>
<h2>Permission denied</h2>
<p>Please <a href="/session_login/">authenticate</a> first. Anonymous blog
posting is not permitted yet. A valid account is required to post new articles.
</p>
<p>Thanks for your understanding and have fun writing stuff... :)</p>
</body>
</html>
    '''
    return HTTPUnauthorized(message , mimetype='text/html')




# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import urllib.request, urllib.parse, urllib.error
import urllib.parse

import gtk
import gobject
from kiwi.component import get_utility
from kiwi.environ import environ
from kiwi.ui.dialogs import BaseDialog, open, save, error, messagedialog
from kiwi.utils import gsignal, type_register

from gazpacho import __version__
from gazpacho import gapi
from gazpacho.actioneditor import GActionsView
from gazpacho.app.bars import bar_manager
from gazpacho.app.dialogs import SingleCloseConfirmationDialog, \
     MultipleCloseConfirmationDialog, UnsupportedWidgetsDialog
from gazpacho.app.preferences import PreferencesDialog
from gazpacho.app.uimstate import WidgetUIMState, ActionUIMState, \
     SizeGroupUIMState
from gazpacho.clipboard import clipboard, ClipboardWindow
from gazpacho.command import AccessorCommand
from gazpacho.commandmanager import command_manager
from gazpacho.config import config
from gazpacho.i18n import _
from gazpacho.interfaces import IPluginManager
from gazpacho.loader.loader import ParseError, ObjectBuilder
from gazpacho.palette import palette
from gazpacho.placeholder import Placeholder
from gazpacho.project import Project
from gazpacho.propertyeditor import PropertyEditor
from gazpacho.sizegroupeditor import SizeGroupView
from gazpacho.stockicons import register_stock_icons
from gazpacho.gadget import Gadget
from gazpacho.widgetview import WidgetTreeView
from gazpacho.widgetregistry import widget_registry
from gazpacho.workspace import WorkSpace

__all__ = ['Application']

# The maximum number of recent items to display
MAX_RECENT_ITEMS = 5

class Application(gobject.GObject):
    """
    Application represents the Gazpacho application itself.

    Signals:
      set-project: a project is set as the current project. This usually
        happens when opening a project.
      close-project: a project is closed
    """
    gsignal('set-project', object),
    gsignal('close-project', object)

    # DND information
    TARGET_TYPE_URI = 100
    targets = [('text/uri-list', 0, TARGET_TYPE_URI)]

    def __init__(self):
        gobject.GObject.__init__(self)

        # The WidgetAdaptor that we are about to add to a container. None if no
        # class is to be added. This also has to be in sync with the depressed
        # button in the Palette
        self._add_class = None

        # This is our current project
        self._project = None
        self._project_counter = 1

        # Merge IDs
        self._recent_menu_uid = -1
        self._open_projects_uid = -1

        # The uim_states maps notebook page numbers to uim states
        self._uim_states = {}
        # The uim state that is currently active
        self._activ_uim_state = None

        # debugging windows
        self._command_stack_window = None
        self._clipboard_window = None

        # here we put views that should update when changing the project
        self._project_views = []

        self._active_view = None
        self._projects = []
        self._show_structure = False

        register_stock_icons()

        self._window = self._application_window_create()

        # Initialize the Plugin Manager
        plugin_manager = get_utility(IPluginManager)
        plugin_manager.load_plugins()
        plugin_manager.activate_plugins(config.get_plugins(), self)

    def _application_window_create(self):
        application_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        application_window.set_name('ApplicationWindow')
        application_window.connect('delete-event', self._delete_event_cb)
        iconfilename = environ.find_resource('pixmap', 'gazpacho-icon.png')
        gtk.window_set_default_icon_from_file(iconfilename)

        # Layout them on the window
        main_vbox = gtk.VBox()
        application_window.add(main_vbox)
        main_vbox.show()

        # Create actions that are always enabled
        bar_manager.add_actions(
            'Normal',
            # File menu
            ('FileMenu', None, _('_File')),
            ('New', gtk.STOCK_NEW, None, None,
             _('New Project'), self._new_cb),
            ('Open', gtk.STOCK_OPEN, None, None,
             _('Open Project'), self._open_cb),
            ('Quit', gtk.STOCK_QUIT, None, None,
             _('Quit'), self._quit_cb),

            # Edit menu
            ('EditMenu', None, _('_Edit')),
            ('Preferences', gtk.STOCK_PREFERENCES, None, None,
             _('Open the preferences dialog'), self._preferences_cb),

            # Object menu
            ('ObjectMenu', None, _('_Objects')),

            # Project menu
            ('ProjectMenu', None, _('_Project')),
            ('ProjectProperties', None, _('_Properties'), None,
             _('Project properties'), self._project_properties_cb),

            # (projects..)

            # Debug menu
            ('DebugMenu', None, _('_Debug')),
            ('DumpData', None, _('Dump data'), '<control>M',
              _('Dump debug data'), self._dump_data_cb),
            ('Preview', None, _('Preview'), None,
             _('Preview current window'), self._preview_cb),

            # Help menu
            ('HelpMenu', None, _('_Help')),
            ('About', gtk.STOCK_ABOUT, None, None, _('About Gazpacho'),
             self._about_cb),
            )

        # Toggle actions
        bar_manager.add_toggle_actions(
            'Normal',
            ('ShowStructure', None, _('Show _structure'), '<control><shift>t',
             _('Show container structure'), self._show_structure_cb, False),
            ('ShowCommandStack', None, _('Show _command stack'), 'F3',
             _('Show the command stack'), self._show_command_stack_cb, False),
            ('ShowClipboard', None, _('Show _clipboard'), 'F4',
             _('Show the clipboard'), self._show_clipboard_cb, False),
            ('ShowWorkspace', None, _('Show _workspace'), '<control><shift>t',
             _('Show container workspace'), self._show_workspace_cb, False),
            )

        # Create actions that reqiuire a project to be enabled
        bar_manager.add_actions(
            'ContextActions',
            # File menu
            ('Save', gtk.STOCK_SAVE, None, None,
             _('Save Project'), self._save_cb),
            ('SaveAs', gtk.STOCK_SAVE_AS, _('Save _As...'), '<shift><control>S',
             _('Save project with different name'), self._save_as_cb),
            ('Close', gtk.STOCK_CLOSE, None, None,
             _('Close Project'), self._close_cb),
            # Edit menu
            ('Undo', gtk.STOCK_UNDO, None, '<control>Z',
             _('Undo last action'), self._undo_cb),
            ('Redo', gtk.STOCK_REDO, None, '<shift><control>Z',
             _('Redo last action'), self._redo_cb)
            )

        bar_manager.add_actions(
            'AlwaysDisabled',
            # Edit menu
            ('Cut', gtk.STOCK_CUT, None, None,
             _('Cut'), None),
            ('Copy', gtk.STOCK_COPY, None, None,
             _('Copy'), None),
            ('Paste', gtk.STOCK_PASTE, None, None,
             _('Paste'), None),
            ('Delete', gtk.STOCK_DELETE, None, '<control>D',
             _('Delete'), None)
            )

        bar_manager.build_interfaces()
        self._update_recent_project_items()
        application_window.add_accel_group(bar_manager.get_accel_group())
        main_vbox.pack_start(bar_manager.get_menubar(), False)
        main_vbox.pack_start(bar_manager.get_toolbar(), False)

        hbox = gtk.HBox(spacing=6)
        main_vbox.pack_start(hbox)
        hbox.show()

        palette.connect('toggled', self._on_palette_toggled)
        hbox.pack_start(palette, False, False)
        palette.show_all()

        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.set_size_request(400, -1)
        hbox.pack_start(sw, True, True)

        self._workspace = WorkSpace()
        sw.add_with_viewport(self._workspace)
        self._workspace.show()
        self._workspace_sw = sw
        self._workspace_action = bar_manager.get_action(
            'ui/MainMenu/EditMenu/ShowWorkspace')

        vpaned = gtk.VPaned()
        vpaned.set_position(150)
        hbox.pack_start(vpaned, True, True)
        vpaned.show()

        notebook = gtk.Notebook()
        vpaned.add1(notebook)
        notebook.show()

        # Widget view
        widget_view = WidgetTreeView(self)
        self._add_view(widget_view)
        page_num = notebook.append_page(widget_view, gtk.Label(_('Widgets')))
        widget_view.show()

        state = WidgetUIMState()
        self._uim_states[page_num] = state

        # Action view
        self.gactions_view = GActionsView(self)
        self._add_view(self.gactions_view)
        page_num = notebook.append_page(self.gactions_view,
                                        gtk.Label(_('Actions')))
        self.gactions_view.show()

        state = ActionUIMState(self.gactions_view)
        self._uim_states[page_num] = state

        # Sizegroup view
        self.sizegroup_view = SizeGroupView(self)
        self._add_view(self.sizegroup_view)
        page_num = notebook.append_page(self.sizegroup_view,
                                        gtk.Label(_('Size Groups')))
        self.sizegroup_view.show()

        state = SizeGroupUIMState(self.sizegroup_view)
        self._uim_states[page_num] = state

        # Add property editor
        self._editor = PropertyEditor(self)
        vpaned.add2(self._editor)
        self._editor.show_all()

        notebook.connect('switch-page', self._on_notebook_switch_page)

        # Statusbar
        statusbar = gtk.Statusbar()
        self._statusbar_menu_context_id = statusbar.get_context_id("menu")
        self._statusbar_actions_context_id = statusbar.get_context_id("actions")
        main_vbox.pack_end(statusbar, False)
        self._statusbar = statusbar
        statusbar.show()

        # dnd doesn't seem to work with Konqueror, at least not when
        # gtk.DEST_DEFAULT_ALL or gtk.DEST_DEFAULT_MOTION is used. If
        # handling the drag-motion event it will work though, but it's
        # a bit tricky.
        application_window.drag_dest_set(gtk.DEST_DEFAULT_ALL,
                                         Application.targets,
                                         gtk.gdk.ACTION_COPY)

        application_window.connect('drag-data-received',
                                   self._dnd_data_received_cb)

        # Enable the current state
        self._active_uim_state = self._uim_states[0]
        self._active_uim_state.enable()

        return application_window

    def _add_recent_project(self, project):
        """
        Add a project to the recent file list. This will also update
        the menu.

        @param project: the project to add
        @type project: L{gazpacho.project.Project}
        """
        if not project.path:
            return

        config.add_recent_project(project.path)
        self._update_recent_project_items()

    def _update_recent_project_items(self):
        """
        Update the list of recent projects in the menu.
        """
        if self._recent_menu_uid:
            bar_manager.remove_ui(self._recent_menu_uid)
            bar_manager.ensure_update()

        ui = ""
        for i, path in enumerate(config.recent_projects):
            if i == MAX_RECENT_ITEMS:
                break

            if not os.path.exists(path):
                config.recent_projects.remove(path)
                continue

            basename = os.path.basename(path)
            if basename.endswith('.glade'):
                basename = basename[:-6]

            ui += '<menuitem action="%s"/>' % basename
            label = '_%d. %s' % (i+ 1, basename)
            action = gtk.Action(basename, label, '', '')
            action.connect('activate', self._open_project_cb, path)
            bar_manager.add_action('RecentProjects', action)

        recent_template = '''<ui>
  <menubar name="MainMenu">
    <menu action="FileMenu">
      <placeholder name="RecentProjects">
      %s
      </placeholder>
    </menu>
  </menubar>
</ui>'''

        self._recent_menu_uid = bar_manager.add_ui_from_string(
            recent_template % ui)

    def _add_view(self, view):
        self._project_views.insert(0, view)
        view.set_project(self._project)
        view.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

    def _confirm_open_project(self, project):
        """
        Ask the user whether to reopen a project that has already been
        loaded (and thus reverting all changes) or just switch to the
        loaded version.

        @param project: the project we might reload
        @type project: L{gazpacho.project.Project}
        @return: True if the project should be reloaded
        @rtype: bool
        """
        submsg1 = _('"%s" is already open.') % project.name
        submsg2 = _('Do you wish to revert your changes and re-open the '
                    'glade file? Your changes will be permanently lost '
                    'if you choose to revert.')
        msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
              (submsg1, submsg2)
        dialog = gtk.MessageDialog(self._window, gtk.DIALOG_MODAL,
                                   gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE,
                                   msg)
        dialog.set_title('')
        dialog.label.set_use_markup(True)
        dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                           gtk.STOCK_REVERT_TO_SAVED, gtk.RESPONSE_YES)
        dialog.set_default_response(gtk.RESPONSE_CANCEL)

        result = dialog.run()
        dialog.destroy()
        return result == gtk.RESPONSE_YES

    def _add_project(self, project):
        # if the project was previously added, don't reload
        for prj in self._projects:
            if prj.path and prj.path == project.path:
                self._set_project(prj)
                return

        self._projects.insert(0, project)

        self._update_open_projects()

        # connect to the project signals so that the editor can be updated
        project.selection.connect_object('selection-changed',
                                         self._on_selection_changed,
                                         project)
        project.connect('changed', self._on_project_changed)
        project.connect('add-gadget', self._on_project_add_gadget)
        project.connect('remove-gadget', self._on_project_remove_gadget)
        project.undo_stack.connect('changed', self._on_undo_stack_changed)

        self._set_project(project)
        self._add_recent_project(project)

    def _update_open_projects(self):
        """
        Update the list of recent projects in the menu.
        """
        if self._open_projects_uid:
            bar_manager.remove_ui(self._open_projects_uid)

        ui = ""
        previous_action = None
        for project in self._projects:
            action = gtk.RadioAction(project.get_id(), project.name,
                                     project.name, '', hash(project))
            if previous_action:
                action.set_group(previous_action)
            action.connect('activate', self._set_project_cb, project)
            bar_manager.add_action('OpenProjects', action)

            ui += '<menuitem action="%s"/>' % action.get_name()
            previous_action = action

        open_projects_template = '''<ui>
  <menubar name="MainMenu">
    <menu action="ProjectMenu">
      <placeholder name="OpenProjects">
      %s
      </placeholder>
    </menu>
  </menubar>
</ui>'''

        self._open_projects_uid = bar_manager.add_ui_from_string(
            open_projects_template % ui)

        bar_manager.set_action_prop('ProjectProperties',
                                    sensitive=len(self._projects))
        bar_manager.ensure_update()

    def _set_project(self, project):
        """
        Set the specified project as the current project.

        @param project: the project
        @type project: gazpacho.project.Project
        """
        if project is self._project:
            return

        if project and project not in self._projects:
            print ('Could not set project because it could not be found '
                   'in the list')
            return

        self._project = project

        for view in self._project_views:
            view.set_project(project)

        for state in self._uim_states.values():
            state.set_project(project)

        self._refresh_title()
        self.refresh_command_stack_view()

        if project:
            # Make sure the radio action is set. This will trigger
            # another call to _set_project but since the project is
            # already set it will just be ignored
            action_name = ('/MainMenu/ProjectMenu/OpenProjects/%s'
                           % project.get_id())
            bar_manager.get_action(action_name).set_active(True)

            # trigger the selection changed signal to update the editor
            self._project.selection.selection_changed()

            # emit this signals so plugins can do their stuff
            self.emit('set-project', project)

        self._update_palette_sensitivity()

    def _refresh_title(self):
        if self._project:
            title = '%s - Gazpacho' % self._project.name
            if self._project.changed:
                title = '*' + title
        else:
            title = 'Gazpacho'
        self.set_title(title)

    def _refresh_project_entry(self, project):
        """Update the project label in the project menu."""
        bar_manager.set_action_prop(project.get_id(), label=project.name)

    def _is_writable(self, filename):
        """
        Check if it is possible to write to the file. If the file
        doesn't exist it checks if it is possible to create it.
        """
        if os.access(filename, os.W_OK):
            return True

        path = os.path.dirname(filename)
        if not os.path.exists(filename) and os.access(path, os.W_OK):
            return True

        return False

    def _save(self, project, path):
        """ Internal save """
        if self._is_writable(path):
            project.save(path)
            self._refresh_project_entry(project)
            self._refresh_title()
            success = True
        else:
            submsg1 = _('Could not save project "%s"') % project.name
            submsg2 = _('Project "%s" could not be saved to %s. '
                        'Permission denied.') % \
                        (project.name, os.path.abspath(path))
            text = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
                   (submsg1, submsg2)
            result = messagedialog(
                gtk.MESSAGE_ERROR, text, parent=self._window,
                buttons=((gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL),
                         (gtk.STOCK_SAVE_AS, gtk.RESPONSE_YES)))

            if result == gtk.RESPONSE_YES:
                success = self._project_save_as(_('Save as'), project)
            else:
                success = False

        self._add_recent_project(project)
        return success

    def _project_save_as(self, title, project):
        filename = save(title, folder=config.lastdirectory)
        if not filename:
            return False

        config.set_lastdirectory(filename)

        return self._save(project, filename)

    def _close_project(self, project):
        """
        Close the project.

        @param project: the project to close
        @type project: L{gazpacho.project.Project}
        """
        project.selection.clear(False)
        if project is self._project:
            project.selection.selection_changed()

        for widget in project.get_widgets():
            widget.destroy()

        bar_manager.remove_ui(-1)
        self._projects.remove(project)

        self.emit('close-project', project)

    def _confirm_close_project(self, project):
        """Show a dialog asking the user whether or not to save the
        project before closing them.

        Return False if the user has chosen not to close the project.
        """
        return self._confirm_close_projects([project])

    def _confirm_close_projects(self, projects):
        """Show a dialog listing the projects and ask whether the user
        wants to save them before closing them.

        Return False if the user has chosen not to close the projects.
        """
        if not projects:
            return True

        if len(projects) == 1:
            dialog = SingleCloseConfirmationDialog(projects[0], self._window)
        else:
            dialog = MultipleCloseConfirmationDialog(projects, self._window)

        ret = dialog.run()
        if ret == gtk.RESPONSE_YES:
            # Go through the chosen projects and save them. Ask for a
            # file name if necessary.
            close = True
            projects_to_save = dialog.get_projects()
            for project in projects_to_save:
                if project.path:
                    saved = self._save(project, project.path)
                else:
                    title = _('Save as')
                    saved = self._project_save_as(title, project)
                    # If the user has pressed cancel we abort everything.
                if not saved:
                    close = False
                    break
        elif ret == gtk.RESPONSE_NO:
            # The user has chosen not to save any projects.
            close = True
        else:
            # The user has cancel the close request.
            close = False

        dialog.destroy()
        return close

    def _set_workspace_title(self, widget):
        self._workspace.set_widget_title(
            widget, '%s - %s' % (gobject.type_name(widget), widget.name))

    # debugging windows

    def _delete_event_for_debugging_window(self, window, event, action):
        # this will hide the window
        action.activate()
        # we don't want the window to be destroyed
        return True

    def _create_debugging_window(self, view, title, action):
        win = gtk.Window()
        win.set_title(title)
        win.set_transient_for(self._window)
        win.add(view)
        view.show_all()
        win.connect('delete-event',
                    self._delete_event_for_debugging_window, action)
        return win

    def _update_palette_sensitivity(self):
        """Update sensitivity of  UI elements (Palette) depending on
        the project list. If the list is empty we should unsensitive
        everything. Otherwise we should set sensitive to True.
        """
        if self._projects:
            palette.set_sensitive(True)
        else:
            palette.set_sensitive(False)


    # Public API

    def new_project(self):
        """
        Create and add a new project.
        """
        project = Project(self)
        project.name = _('Untitled %d') % self._project_counter
        self._project_counter += 1

        self._add_project(project)

    def get_current_project(self):
        """
        Get the currently selected project.

        @return: the project
        @rtype: L{gazpacho.project.Project}
        """
        return self._project

    def get_projects(self):
        """
        Get the open projects.

        @return: list of projects
        @rtype: list
        """
        return self._projects

    def open_project(self, path):
        """
        Open the project at the specified path. If the project has
        already been loaded we ask if we should reload it (close and
        reopen) or just switch to the loaded project.

        @param path: the path to the project file
        @type path: str
        """
        # Check if the project is loaded and ask the user what to do.
        for project in self._projects:
            if project.path and project.path == path:
                if not self._confirm_open_project(project):
                    self._set_project(project)
                    return project

                self._close_project(project)
                break

        project = Project(self)
        try:
            project.load(path)
        except ParseError:
            submsg1 = _('The project could not be loaded')
            submsg2 = _('An error occurred while parsing the file "%s".') % \
                      os.path.abspath(path)
            msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
                      (submsg1, submsg2)
            error(msg)
            self._update_palette_sensitivity()
        except IOError:
            if path in config.recent_projects:
                config.recent_projects.remove(path)
                #self._update_recent_items()

            submsg1 = _('The project could not be loaded')
            submsg2 = _('The file "%s" could not be opened') % \
                      os.path.abspath(path)
            msg = '<span weight="bold" size="larger">%s</span>\n\n%s\n' % \
                      (submsg1, submsg2)
            error(msg)
            self._update_palette_sensitivity()

        failures = project.get_unsupported_widgets()
        if failures:
            dialog = UnsupportedWidgetsDialog(self._window, failures)
            dialog.run()
            dialog.destroy()

        self._add_project(project)
        return project

    def close_current_project(self):
        """
        Close the current project. If there still are open projects
        after the current project has been closed we select one of
        them.
        """
        if not self._project:
            return

        self._close_project(self._project)

        next_project = None
        if self._projects:
            next_project = self._projects[0]

        self._update_open_projects()
        self._set_project(next_project)

    def refresh_command_stack_view(self):
        # Update the command stack view
        if self._command_stack_window is not None:
            command_stack_view = self._command_stack_window.get_child()
            command_stack_view.update()

    def set_title(self, title):
        self._window.set_title(title)

    def refresh_editor(self, prop_name=None):
        self._editor.refresh(prop_name)

    def get_gadget_in_editor(self):
        return self._editor.get_loaded_gadget()

    def create(self, type_name):
        adaptor = widget_registry.get_by_name(type_name)
        gapi.create_gadget(self._project, adaptor, None)

    def run(self):
        """Display the window and run the application"""
        self._window.show()

        gtk.main()

    def validate_widget_names(self, project, show_error_dialogs=True):
        """Return True if all the widgets in project have valid names.

        A widget has a valid name if the name is not empty an unique.
        As soon as this function finds a widget with a non valid name it
        select it in the widget tree and returns False
        """

        widget_names = [widget.name for widget in project.get_widgets()]
        for widget in project.get_widgets():
            # skip internal children (they have None as the name)
            if widget.name is None:
                continue

            if widget.name == '':
                if show_error_dialogs:
                    error(_("There is a widget with an empty name"))
                project.selection.set(widget, True)
                return False

            widget_names.remove(widget.name)

            if widget.name in widget_names:
                if show_error_dialogs:
                    msg = _('The name "%s" is used in more than one widget')
                    error(msg % widget.name)
                project.selection.set(widget, True)
                return False

        return True

    def get_current_context(self):
        """Return the context associated with the current project or None if
        there is not such a project.
        """
        if self._project is None:
            return None
        return self._project.context

    # Properties

    def get_window(self):
        return self._window

    def get_add_class(self):
        return self._add_class
    add_class = property(get_add_class)

    # Callbacks

    def _on_palette_toggled(self, palette):
        """
        When a palette button is clicked one of two things can
        happen. If the button refers to a toplevel widget that widget
        is created and we are done. If it is not a toplevel widget we
        go into a mode where the widget will be added to the first
        placeholder that is selected.
        """
        adaptor = palette.current

        # adaptor may be None if the selector was pressed
        self._add_class = adaptor
        if adaptor and adaptor.is_toplevel():
            gapi.create_gadget(self._project, adaptor, None)

            palette.unselect_widget()
            self._add_class = None

    def _delete_event_cb(self, window, event):
        self._quit_cb()

        # stop other handlers
        return True

    def _on_notebook_switch_page(self, notebook, page, page_num):
        """
        Handler for the notebook's 'switch-page' signal. This will
        disable the current uim state, select a new and enable it
        instead.
        """
        # Disable old state
        self._active_uim_state.disable()

        # Select and enable new state
        state = self._uim_states[page_num]
        self._active_uim_state = state
        state.enable()

    def _on_undo_stack_changed(self, undo_stack):
        """
        Refresh the command stack view when the undo stack has
        changed.
        """
        self.refresh_command_stack_view()

    def _widget_notify_name_cb(self, widget, pspec):
        """
        The workspace title has to be updated when the widget name
        changes.
        """
        self._set_workspace_title(widget)

    def _on_project_add_gadget(self, project, gadget):
        """
        If a toplevel widget has been added it has to be added to the
        workspace as well.

        @param project: the project to which the gadget belongs
        @type project: L{gazpacho.project.Project}
        @param gadget: the gadget that has been added
        @type gadget: L{gazpacho.gadget.Gadget}
        """
        if not self._workspace_action.get_active():
            return

        if isinstance(gadget.widget, gtk.Window):
            gadget.widget.connect('notify::name', self._widget_notify_name_cb)
            self._workspace.add(gadget.widget)
            self._set_workspace_title(gadget.widget)

    def _on_project_remove_gadget(self, project, gadget):
        """
        If a toplevel widget has been removed from the project it has
        to be removed from the workspace as well.

        @param project: the project to which the gadget belongs
        @type project: L{gazpacho.project.Project}
        @param gadget: the gadget that has been removed
        @type gadget: L{gazpacho.gadget.Gadget}
        """
        if not self._workspace_action.get_active():
            return

        if isinstance(gadget.widget, gtk.Window):
            self._workspace.remove(gadget.widget)

    def _on_project_changed(self, project):
        """
        Refresh the title window title. This is done is done to show
        if the project has been modified since it was last saved.

        @param project: the project that has been modified
        @type project: L{gazpacho.project.Project}
        """
        self._refresh_title()

    def _on_selection_changed(self, project):
        """
        The widget selection has changed, switch to the project for
        which the selection belong and update the property editor.
        """
        # Set the sensitivity of the preview menu item
        bar_manager.set_action_prop('Preview',
                                    sensitive=(len(project.selection) == 1))

        # Decide if we have to switch to another project
        if self._project != project:
            self._set_project(project)
            return

        if self._editor:
            children = self._project.selection
            if len(children) == 1 and not isinstance(children[0], Placeholder):
                self._editor.display(Gadget.from_widget(children[0]))
            else:
                self._editor.hide_content()

    def _set_project_cb(self, action, project):
        """
        Set the correct project when the project menu item is
        selected.

        @param action: the action that triggered the event
        @type action: gtk.RadioAction
        @param project: the selected project
        @type project: L{gazpacho.project.Project}
        """
        # ignore the project that has been deselected
        if action.get_active():
            self._set_project(project)

    def _open_project_cb(self, action, path):
        self.open_project(path)

    def _create_command_view(self):
        from gazpacho.commandview import CommandView
        view = CommandView()
        self._add_view(view)

        title = _('Command Stack')
        action = bar_manager.get_action(
            '/MainMenu/DebugMenu/ShowCommandStack')
        self._command_stack_window = self._create_debugging_window(view,
                                                                   title,
                                                                   action)

    def _show_command_stack_cb(self, action):
        if self._command_stack_window is None:
            self._create_command_view()
            self._command_stack_window.show()
        else:
            if self._command_stack_window.get_property('visible'):
                self._command_stack_window.hide()
            else:
                self._command_stack_window.show_all()

    def _show_clipboard_cb(self, action):
        """Show/hide the clipboard window."""
        if self._clipboard_window is None:
            action = bar_manager.get_action(
                '/MainMenu/DebugMenu/ShowClipboard')
            self._clipboard_window = ClipboardWindow(self._window, clipboard)
            self._clipboard_window.connect(
                'delete-event', self._delete_event_for_debugging_window,
                action)
            self._clipboard_window.show_window()
        else:
            if self._clipboard_window.get_property('visible'):
                self._clipboard_window.hide_window()
            else:
                self._clipboard_window.show_window()

    def _dnd_data_received_cb(self, widget, context, x, y, data, info, time):
        """Callback that handles drag 'n' drop of glade files."""
        if info != Application.TARGET_TYPE_URI:
            return

        for uri in data.data.split('\r\n'):
            uri_parts = urllib.parse.urlparse(uri)
            if uri_parts[0] == 'file':
                path = urllib.request.url2pathname(uri_parts[2])
                self.open_project(path)

    # File action callbacks

    def _new_cb(self, action):
        self.new_project()

    def _open_cb(self, action):
        filename = open(parent=self._window, patterns=['*.glade'],
                        folder=config.lastdirectory)
        if filename:
            self.open_project(filename)
            config.set_lastdirectory(filename)

    def _save_cb(self, action):
        project = self._project

        # check that all the widgets have valid names
        if not self.validate_widget_names(project):
            return

        if project.path is not None:
            self._save(project, project.path)
            return

        # If instead we don't have a path yet, fire up a file chooser
        self._save_as_cb(None)

    def _save_as_cb(self, action):
        if action is None:
            # we were called from the _save_cb callback
            title = _('Save')
        else:
            title = _('Save as')
        self._project_save_as(title, self._project)

    def _close_cb(self, action):
        if not self._project:
            return

        if self._project.changed:
            close = self._confirm_close_project(self._project)
            if not close:
                return

        self.close_current_project()

    def _quit_cb(self, action=None):
        unsaved_projects = [p for p in self._projects if p.changed]
        close = self._confirm_close_projects(unsaved_projects)
        if not close:
            return

        config.save()

        plugin_manager = get_utility(IPluginManager)
        plugin_manager.deactivate_all()
        gtk.main_quit()

    # Edit action callbacks

    def _undo_cb(self, action):
        command_manager.undo(self._project)

    def _redo_cb(self, action):
        command_manager.redo(self._project)

    def _show_structure_cb(self, action):
        self._show_structure = not self._show_structure
        for project in self._projects:
            for widget in project.get_widgets():
                if isinstance(widget, gtk.Window):
                    widget.queue_draw()

    def _show_workspace_cb(self, action):
        workspace = self._workspace_sw
        if action.get_active():
            workspace.show()
        else:
            workspace.hide()

    def _preferences_cb(self, action):
        dialog = PreferencesDialog()
        dialog.run()

    # Project action callbacks

    def _project_properties_cb(self, action):
        # XXX: Use a glade file here, pending #329914 and #330652
        project = self.get_current_project()

        b = BaseDialog(self._window, _('Project properties'))
        b.set_resizable(False)
        b.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)

        hbox = gtk.HBox(spacing=6)
        b.vbox.pack_start(hbox, False, False)
        hbox.show()

        label = gtk.Label()
        label.set_markup_with_mnemonic(_('_Translation domain:'))
        hbox.pack_start(label, False, False)
        label.show()

        entry = gtk.Entry()
        entry.connect('activate', lambda entry: b.response(gtk.RESPONSE_CLOSE))
        entry.set_text(project.get_domain())
        hbox.pack_start(entry, False, False)
        entry.show()
        b.vbox.show_all()

        response = b.run()
        if response == gtk.RESPONSE_CLOSE:
            domain = entry.get_text()
            if domain != project.get_domain():
                cmd = AccessorCommand(_('Set domain to %s') % domain,
                                      domain,
                                      project.set_domain,
                                      project.get_domain)
                command_manager.execute(cmd, project)
        b.destroy()

    # Debug action callbacks

    def _dump_data_cb(self, action):
        """This method is only useful for debugging.

        Any developer can print whatever he/she wants here
        the only rule is: clean everything before you commit it.

        This will be called upon CONTROL+M or by using the menu
        item Debug/Dump Data
        """
        project = self.get_current_project()
        print(project.serialize()[:-1])

        def dump_widget(widget, lvl=0):
            print('%s %s (%s)' % (' ' * lvl,
                                  gobject.type_name(widget),
                                  widget.get_name()))
            if isinstance(widget, gtk.Container):
                for child in widget.get_children():
                    dump_widget(child, lvl+1)

        for widget in project.get_widgets():
            if widget.get_parent():
                continue
            dump_widget(widget)

    def _preview_cb(self, action):
        project = self.get_current_project()
        if not project.selection:
            return

        xml = project.serialize()
        builder = ObjectBuilder(buffer=xml)
        toplevel = project.selection[0].get_toplevel()
        widget = builder.get_widget(toplevel.get_name())
        widget.show_all()

    # Help action callbacks

    def _about_cb(self, action):
        about = gtk.AboutDialog()
        about.set_name('Gazpacho')
        about.set_version(__version__)
        authorsfile = file(environ.find_resource('doc', 'AUTHORS'))
        authors = [a.strip() for a in authorsfile.readlines()]
        authors.append('') # separate authors from contributors
        contributorsfile = file(environ.find_resource('doc', 'CONTRIBUTORS'))
        authors.extend([c.strip() for c in contributorsfile.readlines()[:-2]])
        about.set_authors(authors)
        license = file(environ.find_resource('doc', 'COPYING')).read()
        about.set_license(license)
        about.set_website('http://gazpacho.sicem.biz')
        about.run()
        about.destroy()

    # Do not add anything here, add it above in the appropriate section

type_register(Application)



from livesettings import *
from django.utils.translation import ugettext_lazy as _

# this is so that the translation utility will pick up the string
gettext = lambda s: s

PAYMENT_GROUP = ConfigurationGroup('PAYMENT_DUMMY', 
    _('Payment Test Module Settings'), 
    ordering = 100)

config_register_list(
    BooleanValue(PAYMENT_GROUP, 
        'SSL', 
        description=_("Use SSL for the module checkout pages?"), 
        default=False),
        
    BooleanValue(PAYMENT_GROUP, 
        'LIVE', 
        description=_("Accept real payments"),
        help_text=_("False if you want to be in test mode"),
        default=False),
        
    ModuleValue(PAYMENT_GROUP,
        'MODULE',
        description=_('Implementation module'),
        hidden=True,
        default = 'payment.modules.dummy'),
        
    StringValue(PAYMENT_GROUP,
        'KEY',
        description=_("Module key"),
        hidden=True,
        default = 'DUMMY'),

    StringValue(PAYMENT_GROUP,
        'LABEL',
        description=_('English name for this group on the checkout screens'),
        default = 'Payment test module',
        help_text = _('This will be passed to the translation utility')),

    StringValue(PAYMENT_GROUP,
        'URL_BASE',
        description=_('The url base used for constructing urlpatterns which will use this module'),
        default = '^dummy/'),

    MultipleStringValue(PAYMENT_GROUP,
        'CREDITCHOICES',
        description=_('Available credit cards'),
        choices = (
            (('Visa','Visa')),
            (('Mastercard','Mastercard')),
            (('Discover','Discover')),
            (('American Express', 'American Express'))),
        default = ('Visa', 'Mastercard', 'Discover', 'American Express')),
        
    BooleanValue(PAYMENT_GROUP,
        'CAPTURE',
        description=_('Capture Payment immediately?'),
        default=True,
        help_text=_('IMPORTANT: If false, a capture attempt will be made when the order is marked as shipped."')),
        
    BooleanValue(PAYMENT_GROUP,
        'AUTH_EARLY',
        description=_("Early AUTH"),
        help_text=_("Authenticate on the card entry page, causes an immediate $.01 AUTH and release, allowing errors with the card to show on the card entry page."),
        default=False),

    BooleanValue(PAYMENT_GROUP,
        'EXTRA_LOGGING',
        description=_("Verbose logs"),
        help_text=_("Add extensive logs during post."),
        default=False)
)



"""Urls which need to be loaded at root level."""
from django.conf.urls.defaults import *

urlpatterns = patterns('',
    (r'^admin/product/configurableproduct/(?P<id>\d+)/getoptions/', 
        'product.views.get_configurable_product_options', {}, 
        'configurableproduct'),
)

urlpatterns += patterns('product.views.adminviews',
    (r'^admin/inventory/edit/$', 
        'edit_inventory', {}, 'edit_inventory'),
    (r'^inventory/export/$',
        'export_products', {}, 'product_export'),
    (r'^inventory/import/$', 
        'import_products', {}, 'product_import'),
    # (r'^inventory/report/$', 
    #     'product_active_report', {}, 'satchmo_admin_product_report'),
    (r'^admin/(?P<product_id>\d+)/variations/$', 
        'variation_manager', {}, 'variation_manager'),
    (r'^admin/variations/$', 
        'variation_list', {}, 'variation_list'),
)



from django.db import models
from payment.config import credit_choices, labelled_gateway_choices

class CreditChoiceCharField(models.CharField):

    def __init__(self, choices="__DYNAMIC__", *args, **kwargs):
        if choices == "__DYNAMIC__":
            kwargs['choices'] = credit_choices()

        super(CreditChoiceCharField, self).__init__(*args, **kwargs)

class PaymentChoiceCharField(models.CharField):
    
    def __init__(self, choices="__DYNAMIC__", *args, **kwargs):
        if choices == "__DYNAMIC__":
            kwargs['choices'] = labelled_gateway_choices()
                    
        super(PaymentChoiceCharField, self).__init__(*args, **kwargs)



from django.utils.translation import ugettext_lazy as _
from livesettings import *

SHIP_MODULES = config_get('SHIPPING', 'MODULES')

# No need to add the choice, since it is in by default
# SHIP_MODULES.add_choice(('shipping.modules.per', _('Per piece')))

SHIPPING_GROUP = config_get_group('SHIPPING')

config_register_list(

    DecimalValue(SHIPPING_GROUP,
        'PER_RATE',
        description=_("Per item price"),
        requires=SHIP_MODULES,
        requiresvalue='shipping.modules.per',
        default="4.00"),

    StringValue(SHIPPING_GROUP,
        'PER_SERVICE',
        description=_("Per Item Shipping Service"),
        help_text=_("Shipping service used with per item shipping"),
        requires=SHIP_MODULES,
        requiresvalue='shipping.modules.per',
        default=u"U.S. Mail"),

    StringValue(SHIPPING_GROUP,
        'PER_DAYS',
        description=_("Per Item Delivery Days"),
        requires=SHIP_MODULES,
        requiresvalue='shipping.modules.per',
        default="3 - 4 business days")
)



from django.conf.urls.defaults import *
from livesettings import config_get_group

config = config_get_group('PAYMENT_GOOGLE')

urlpatterns = patterns('',
     (r'^$', 'payment.modules.google.views.pay_ship_info', {'SSL': config.SSL.value}, 'GOOGLE_satchmo_checkout-step2'),
     (r'^confirm/$', 'payment.modules.google.views.confirm_info', {'SSL': config.SSL.value}, 'GOOGLE_satchmo_checkout-step3'),
     (r'^success/$', 'payment.modules.google.views.success', {'SSL': config.SSL.value}, 'GOOGLE_satchmo_checkout-success'),
     (r'^notification/$', 'payment.modules.google.views.notification', {'SSL': config.SSL.value},
        'GOOGLE_satchmo_checkout-notification'),
     (r'^confirmorder/$', 'payment.views.confirm.confirm_free_order', 
        {'SSL' : config.SSL.value, 'key' : 'GOOGLE'}, 'GOOGLE_satchmo_checkout_free-confirm')
)



# -*- coding: UTF-8 -*-
from decimal import Decimal
from django.contrib.sites.models import Site
from django.test import TestCase
from keyedcache import cache_delete
from l10n.models import Country
from livesettings import config_get_group, config_value
from models import *
from product.models import Product
from satchmo_store.contact.models import AddressBook, Contact, ContactRole
from satchmo_store.shop.models import Order, OrderItem, OrderItemDetail
from utils import generate_certificate_code, generate_code
import datetime, logging

log = logging.getLogger('giftcertificate.tests')

alphabet = 'abcdefghijklmnopqrstuvwxyz'

def make_test_order(country, state):
    c = Contact(first_name="Gift", last_name="Tester", 
        role=ContactRole.objects.get(pk='Customer'), email="gift@example.com")
    c.save()
    if not isinstance(country, Country):
        country = Country.objects.get(iso2_code__iexact = country)
    ad = AddressBook(contact=c, description="home",
        street1 = "test", state=state, city="Portland",
        country = country, is_default_shipping=True,
        is_default_billing=True)
    ad.save()
    site = Site.objects.get_current()
    o = Order(contact=c, shipping_cost=Decimal('0.00'), site=site)
    o.save()

    p = Product.objects.get(slug='GIFT10')
    price = p.unit_price
    log.debug("creating with price: %s", price)
    item1 = OrderItem(order=o, product=p, quantity='2.0',
        unit_price=price, line_item_price=price*2)
    item1.save()

    detl = OrderItemDetail(name = 'email', value='me@example.com', sort_order=0, item=item1)
    detl.save()
    detl = OrderItemDetail(name = 'message', value='hello there', sort_order=0, item=item1)
    detl.save()

    return o

class TestGenerateCode(TestCase):

    def testGetCode(self):
        c = generate_code(alphabet, '^^^^')
        
        self.assertEqual(len(c), 4)
        
        for ch in c:
            self.assert_(ch in alphabet)
            
    def testGetCode2(self):
        c = generate_code(alphabet, '^^^^-^^^^')
        c2 = generate_code(alphabet, '^^^^-^^^^')
        self.assertNotEqual(c,c2)
        
    def testFormat(self):
        c = generate_code(alphabet, '^-^-^-^')
        for i in (0,2,4,6):
            self.assert_(c[i] in alphabet)
        for i in (1,3,5):
            self.assertEqual(c[i], '-')

class TestGenerateCertificateCode(TestCase):
    def setUp(self):
        self.charset = config_value('PAYMENT_GIFTCERTIFICATE', 'CHARSET')
        self.format = config_value('PAYMENT_GIFTCERTIFICATE', 'FORMAT')
        
    def testGetCode(self):
        c = generate_certificate_code()
        self.assertEqual(len(c), len(self.format))

        chars = [x for x in self.format if not x=='^']
        chars.extend(self.charset)
        for ch in c:
            self.assert_(ch in chars)
    
class TestCertCreate(TestCase):
    fixtures = ['l10n-data.yaml','test_shop']
    
    def setUp(self):
        self.site = Site.objects.get_current()
    
    def tearDown(self):
        cache_delete()

    def testCreate(self):
        gc = GiftCertificate(start_balance = '100.00', site=self.site)
        gc.save()
        
        self.assert_(gc.code)
        self.assertEqual(gc.balance, Decimal('100.00'))

    def testUse(self):
        gc = GiftCertificate(start_balance = '100.00', site=self.site)
        gc.save()
        bal = gc.use('10.00')
        self.assertEqual(bal, Decimal('90.00'))
        self.assertEqual(gc.usages.count(), 1)
        
        
class GiftCertOrderTest(TestCase):

    fixtures = ['l10n-data.yaml', 'test_shop.yaml', 'test_giftcertificate.yaml', 'test_giftcertificate_config.yaml']
    
    def tearDown(self):
        cache_delete()

    def testOrderSuccess(self):
        """Test cert creation on order success"""
        cache_delete()
        order = make_test_order('US', '')
        order.order_success()
    
        certs = order.giftcertificates.all()
        self.assertEqual(len(certs), 1)
        c = certs[0]
        self.assertEqual(c.balance, Decimal('20.00'))
        self.assertEqual(c.recipient_email, 'me@example.com')
        self.assertEqual(c.message, 'hello there')
        



import sys
#import mainapp
from notmm.utils.wsgilib import HTTPRequest
from test_support import (
    WSGIControllerTestCase, 
    TestClient, 
    unittest,
    make_app,
    settings,
    ResponseClass
    )

class MessageTestCase(WSGIControllerTestCase):
    def test_create_message(self):
        pass




"""
URLConf for Django user registration.

Recommended usage is to use a call to ``include()`` in your project's
root URLConf to include this URLConf for any URL beginning with
'/accounts/'.

"""
from django.conf.urls.defaults import *
from livesettings import config_value

# extending the urls in contacts
from satchmo_store.contact.urls import urlpatterns
from signals_ahoy.signals import collect_urls
from satchmo_store import accounts


# Activation keys get matched by \w+ instead of the more specific
# [a-fA-F0-9]+ because a bad activation key should still get to the view;
# that way it can return a sensible "invalid key" message instead of a
# confusing 404.
urlpatterns += patterns('satchmo_store.accounts.views',
    (r'^activate/(?P<activation_key>\w+)/$', 'activate', {}, 'registration_activate'),
    (r'^login/$', 'emaillogin', {'template_name': 'registration/login.html'}, 'auth_login'),
    (r'^register/$', 'register', {}, 'registration_register'),
    (r'^secure/login/$', 'emaillogin', {'SSL' : True, 'template_name': 'registration/login.html'}, 'auth_secure_login'),    
)

urlpatterns += patterns('',
    ('^logout/$','django.contrib.auth.views.logout', {'template_name': 'registration/logout.html'}, 'auth_logout'),
    )

verify = (config_value('SHOP', 'ACCOUNT_VERIFICATION') == 'EMAIL')

urlpatterns += patterns('django.views.generic',
    (r'^register/complete/$', 'simple.direct_to_template',
        {'template': 'registration/registration_complete.html',
        'extra_context': {'verify': verify }}, 
        'registration_complete'),
)

#Dictionary for authentication views
password_reset_dict = {
    'template_name': 'registration/password_reset_form.html',
    'email_template_name': 'registration/password_reset.txt',
}

# the "from email" in password reset is problematic... it is hard coded as None
urlpatterns += patterns('django.contrib.auth.views',
    (r'^password_reset/$', 'password_reset', password_reset_dict, 'auth_password_reset'),
    (r'^password_reset/done/$', 'password_reset_done', {'template_name':'registration/password_reset_done.html'}, 'auth_password_reset_done'),
    (r'^password_change/$', 'password_change', {'template_name':'registration/password_change_form.html'}, 'auth_password_change'),
    (r'^password_change/done/$', 'password_change_done', {'template_name':'registration/password_change_done.html'}, 'auth_change_done'),
    (r'^reset/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', 'password_reset_confirm'),
    (r'^reset/done/$', 'password_reset_complete'),
)

collect_urls.send(sender=accounts, patterns=urlpatterns)



from django.conf.urls.defaults import *
from livesettings import config_get_group

config = config_get_group('PAYMENT_PROTX')

urlpatterns = patterns('',
     (r'^$', 'payment.modules.protx.views.pay_ship_info', 
        {'SSL':config.SSL.value}, 'PROTX_satchmo_checkout-step2'),

     (r'^confirm/$', 'payment.modules.protx.views.confirm_info', 
        {'SSL':config.SSL.value}, 'PROTX_satchmo_checkout-step3'),

    (r'^secure3d/$', 'payment.modules.protx.views.confirm_secure3d', 
       {'SSL':config.SSL.value}, 'PROTX_satchmo_checkout-secure3d'),

     (r'^success/$', 'payment.views.checkout.success', 
        {'SSL':config.SSL.value}, 'PROTX_satchmo_checkout-success'),
)


from notmm.utils.urlmap import RegexURLMap
from django.conf.urls import url
from store.localsite import views

urlpatterns = RegexURLMap()

urlpatterns.add_routes('',
    url(r'example/', views.example, {}),
)



from notmm.utils.urlmap import RegexURLMap

urlpatterns = RegexURLMap()

#urlpatterns.set_default_prefix('/foo/')

urlpatterns.add_routes('sandbox_sqlalchemy.views', 
        ### decorators tests
        ### with_schevo_database test
        (r'^test_with_session$', 'test_with_session', {
            'template_name' : 'sandbox/test.mako'
        }),
)




def active_gateways():
    """Get a list of activated payment gateways, in the form of
    [(module, config module name),...]
    """
    from django.db import models
    gateways = []
    for app in models.get_apps():
        if hasattr(app, 'PAYMENT_PROCESSOR'):
            parts = app.__name__.split('.')[:-1]
            module = ".".join(parts)
            group = 'PAYMENT_%s' % parts[-1].upper()
            gateways.append((module, group))
    return gateways



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import django.forms as forms

__all__ = ('LoginForm',)

class LoginForm(forms.Form):
    """Basic login form for generic User authentication/authorization"""
    username = forms.CharField(label="Username", max_length=30)
    password = forms.CharField(label="Password", widget=forms.PasswordInput)
#



# Copyright (C) 2005 by SICEm S.L. and Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
from xml.dom import minidom

import gtk

from gazpacho.propertyeditor import PropertyCustomEditorWithDialog
from gazpacho.i18n import _
from gazpacho.properties import prop_registry, TransparentProperty, StringType
from gazpacho.uieditor import UIEditor
from gazpacho.gadget import Gadget
from gazpacho.gaction import GActionGroup
from gazpacho.widgets.base.base import ContainerAdaptor
from gazpacho.placeholder import Placeholder

class UIPropEditor(PropertyCustomEditorWithDialog):
    dialog_class = UIEditor
    button_text = _('Edit UI Definition...')

class UIAdaptor(TransparentProperty, StringType):
    """This represents a fake property to edit the ui definitions of
    menubars and toolbars.

    It does not store anything because this information is stored in
    the uimanager itself. It's just a way to put a button in the Gazpacho
    interface to call the UIEditor.
    """
    custom_editor = UIPropEditor

class MenuBarUIAdapter(UIAdaptor):
    pass

#prop_registry.override_property('GtkMenuBar::ui', MenuBarUIAdapter)

class ToolbarUIAdapter(UIAdaptor):
    pass

prop_registry.override_property('GtkToolbar::ui', ToolbarUIAdapter)

class CommonBarsAdaptor(ContainerAdaptor):

    def post_create(self, context, widget, ui_string):
        # create some default actions
        gadget = Gadget.from_widget(widget)
        project = gadget.project

        # create default action group if it doesn't exist
        project = context.get_project()
        if project.uim.get_action_group('DefaultActions') is None:
            project.add_action_group(GActionGroup('DefaultActions'))

        project.uim.add_ui(gadget, ui_string)
        new_widget = project.uim.get_widget(gadget)

        # we need to replace widget with new_widget
        gadget.setup_widget(new_widget)
        #gadget.apply_properties()

    def save(self, context, gadget):
        """This saver is needed to avoid saving the children of toolbars
        and menubars
        """

        project = context.get_project()
        if project.get_version() == 'gtkbuilder':
            constructor = project.uim.get_name()
        else:
            constructor = 'initial-state'
        gadget.constructor = constructor

    def load(self, context, widget):
        """This loader is special because of these features:
        - It does not load the children of the menubar/toolbar
        - Load the uimanager and put its content (action groups) into the
        project
        """

#         # we need to save the properties of this widget because otherwise
#         # when we got it from the uimanager it's gonna be another widget with
#         # different properties
#         props = {}
#         for prop in gobject.list_properties(widget):
#             if 1 or prop.flags != gobject.PARAM_READWRITE:
#                 continue
#             if propertyclass.get_type_from_spec(prop) is gobject.TYPE_OBJECT:
#                 continue
#             # FIXME: This need to use the values from the catalog.
#             # But it doesn't work right now, the property in
#             # klass.properties is always set to False.
#             if prop.name == 'parent' or prop.name == 'child':
#                 continue
#             props[prop.name] = widget.get_property(prop.name)

        project = context.get_project()

        old_name = widget.name
        gadget = Gadget.load(widget, project)
        gadget._name = gadget.widget.name

        # change the widget for the one we get from the uimanager
        project.uim.load_widget(gadget, old_name)

        return gadget

    def fill_empty(self, context, widget):
        pass

    def delete(self, context, widget):
        gadget = Gadget.from_widget(widget)
    
        # replace the widget with a placeholder
        parent = gadget.get_parent()
        placeholder = None
        if parent:
            # since the uim widget will be removed we have to replace
            # it with a placeholder in order to know it's location
            placeholder = Placeholder()
            placeholder.set_name(gadget.name)
            Gadget.replace(gadget.widget, placeholder , parent)
        
        uim = context.get_project().uim
        ui_defs = uim.get_ui(gadget)
        uim.remove_gadget(gadget)
    
        if placeholder:
            # the placeholder has to be associated with the gadget,
            # this is a bit ugly
            gadget.widget = placeholder
            placeholder.set_data('gazpacho::gadget', gadget)
    
        return ui_defs, placeholder
    
    def restore(self, context, widget, data):
        ui_defs, placeholder = data    
        gadget = Gadget.from_widget(widget)
        
        # adding the widget to uim will replace the gtk-widget
        # connected to the gadget
        context.get_project().uim.add_gadget(gadget, ui_defs)
    
        if placeholder:
            # replace the placeholder with the real uim widget
            Gadget.replace(placeholder, gadget.widget, gadget.get_parent())
        


from gazpacho.widgets.base.menueditor.widgets import MenuBar

class MenuBarEditingInfo(object):
    def __init__(self):
        self.editing = False
        self.edit_activate_id = 0
        self.fakeBar = None

class MenuBarAdaptor(ContainerAdaptor):
    """Adaptor that allow in-place editing of MenuBars

    The basic idea of this adaptor is to put an 'Edit' button in each
    menu bar. When the user clicks on this button the menubar is replaced
    by a custom widget that looks like a menubar but it is not. It's a
    menueditor.widgets.MenuBar widget that allow in-place edition of a
    menubar.

    When the user selects another widget of the project the fake menubar
    is replaced again by the real one.
    """

    def __init__(self, *args, **kwargs):
        super(MenuBarAdaptor, self).__init__(*args, **kwargs)
        self._bars = {}

    def post_create(self, context, menuBar, interactive):
        # create default action group if it doesn't exist
        project = context.get_project()
        if project.uim.get_action_group('DefaultActions') is None:
            project.add_action_group(GActionGroup('DefaultActions'))

        # Add the basic UI definition
        gadget = Gadget.from_widget(menuBar)
        name = gadget.name
        ui_string = '<menubar action="%s" name="%s"/>' % (name, name)
        project.uim.add_ui(gadget, ui_string)
        new_widget = project.uim.get_widget(gadget)
        self._add_edit_button(new_widget)

        # we need to replace widget with new_widget
        gadget.setup_widget(new_widget)

        self._bars[new_widget] = MenuBarEditingInfo()

    def fill_empty(self, context, widget):
        pass

    def _add_edit_button(self, widget):
        item = gtk.ImageMenuItem(stock_id=gtk.STOCK_EDIT)
        item.set_right_justified(True)
        item.show_all()
        widget.append(item)
        item.connect('button-press-event', self._on_edit_press_event, widget)

    def _on_edit_press_event(self, item, event, menuBar):
        self._start_editing(menuBar)

    def _start_editing(self, menuBar):
        gadget = Gadget.from_widget(menuBar)
        project = gadget.project

        # create the fake menubar
        fakeBar = MenuBar(gadget)

        # load it with our xml ui definition
        ui_def = project.uim.get_ui(gadget, 'initial-state')
        if ui_def:
            doc = minidom.parseString(ui_def[0])
            fakeBar.load(doc.documentElement)

        # replace the real one with this fake one
        fakeBar.show_all()
        self.replace_child(project.context, menuBar, fakeBar, menuBar.parent)

        # save some information to restore the state in _end_editing
        self._bars[menuBar].editing = True
        self._bars[menuBar].fakeBar = fakeBar
        i = project.selection.connect('selection-changed',
                                      self._on_selection__changed, menuBar)
        self._bars[menuBar].edit_activate_id = i

        fakeBar.get_toplevel().queue_draw()

    def _end_editing(self, menuBar):
        gadget = Gadget.from_widget(menuBar)
        if not gadget:
            return
        project = gadget.project
        fakeBar = self._bars[menuBar].fakeBar
        parent = fakeBar.parent

        # put the real menuBar back in its place
        newBar = project.uim.get_widget(gadget)
        fakeBar.hide_all() # it's very important to hide this widget now
        newBar.show_all()
        self.replace_child(project.context, fakeBar, newBar, parent)

        # restore the state
        i = self._bars[menuBar].edit_activate_id
        project.selection.disconnect(i)
        self._bars[menuBar].editing = False
        self._bars[menuBar].fakeBar = None
        self._bars[menuBar].edit_activate_id = 0

        if newBar != menuBar:
            self._add_edit_button(newBar)
            project.remove_widget(menuBar)
            gadget.setup_widget(newBar)
            project.add_widget(newBar)

            info = self._bars[menuBar]
            del self._bars[menuBar]
            self._bars[newBar] = info
            menuBar.destroy()

    def _on_selection__changed(self, selection, menuBar):
        is_editing = self._bars[menuBar].editing
        found = False
        for widget in selection:
            if widget == menuBar:
                found = True
                break

        if not found and is_editing:
            self._end_editing(menuBar)

    # The save and load methods are taken from the CommonsBarAdaptor
    # They should be refactored again when we change the Toolbar adaptor
    def save(self, context, gadget):
        """This saver is needed to avoid saving the children of toolbars
        and menubars
        """

        project = context.get_project()
        if project.get_version() == 'gtkbuilder':
            constructor = project.uim.get_name()
        else:
            constructor = 'initial-state'
        gadget.constructor = constructor

    def load(self, context, widget):
        """This loader is special because of these features:
        - It does not load the children of the menubar/toolbar
        - Load the uimanager and put its content (action groups) into the
        project
        """

#         # we need to save the properties of this widget because otherwise
#         # when we got it from the uimanager it's gonna be another widget with
#         # different properties
#         props = {}
#         for prop in gobject.list_properties(widget):
#             if 1 or prop.flags != gobject.PARAM_READWRITE:
#                 continue
#             if propertyclass.get_type_from_spec(prop) is gobject.TYPE_OBJECT:
#                 continue
#             # FIXME: This need to use the values from the catalog.
#             # But it doesn't work right now, the property in
#             # klass.properties is always set to False.
#             if prop.name == 'parent' or prop.name == 'child':
#                 continue
#             props[prop.name] = widget.get_property(prop.name)

        project = context.get_project()

        old_name = widget.name
        gadget = Gadget.load(widget, project)
        gadget._name = gadget.widget.name

        # change the widget for the one we get from the uimanager
        project.uim.load_widget(gadget, old_name)

        # add the edit button
        self._add_edit_button(gadget.widget)

        # add it to our bars dictionary
        if gadget.widget not in list(self._bars.keys()):
            self._bars[gadget.widget] = MenuBarEditingInfo()

        return gadget

    # XXX this is the same as for toolbar and should be in a common base class

    def delete(self, context, widget):
        gadget = Gadget.from_widget(widget)
    
        # replace the widget with a placeholder
        parent = gadget.get_parent()
        placeholder = None
        if parent:
            placeholder = Placeholder()
            placeholder.set_name(gadget.name)
            Gadget.replace(gadget.widget, placeholder , parent)
        
        uim = context.get_project().uim
        ui_defs = uim.get_ui(gadget)
        uim.remove_gadget(gadget)
    
        if placeholder:
            gadget.widget = placeholder
            placeholder.set_data('gazpacho::gadget', gadget)
    
        return ui_defs, placeholder
    
    def restore(self, context, widget, data):
        ui_defs, placeholder = data    
        gadget = Gadget.from_widget(widget)
        
        # adding the widget to uim will replace the gtk-widget
        # connected to the gadget
        context.get_project().uim.add_gadget(gadget, ui_defs)
    
        if placeholder:
            Gadget.replace(placeholder, gadget.widget, gadget.get_parent())


class ToolbarAdaptor(CommonBarsAdaptor):
    def post_create(self, context, toolbar, interactive=True):
        gadget = Gadget.from_widget(toolbar)
        ui_string = '<toolbar action="%s" name="%s"/>' % (gadget.name,
                                                          gadget.name)

        super(ToolbarAdaptor, self).post_create(context, toolbar, ui_string)

